import django
from django.db.models.fields import FieldDoesNotExist
from django.utils.translation import get_language, ugettext_lazy as _l
from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from hvad.exceptions import WrongManager
from hvad.utils import get_cached_translation, set_cached_translation, load_translation
from hvad.contrib.restframework.utils import TranslationListSerializer
from collections import OrderedDict

veto_fields = ('id', 'master')


#=============================================================================

class NestedTranslationSerializer(serializers.ModelSerializer):
    def to_representation(self, instance):
        """ Nested serializer gets the full object, but some field serializers need the
            actual model their data lives in, that is, the translation. We detect it here. """
        translation = get_cached_translation(instance)
        ret = OrderedDict()
        for field in self._readable_fields:
            try:
                try:
                    attribute = field.get_attribute(instance)
                except (WrongManager, FieldDoesNotExist):
                    attribute = field.get_attribute(translation)
            except SkipField:
                continue
            if attribute is None:
                ret[field.field_name] = None
            else:
                ret[field.field_name] = field.to_representation(attribute)
        return ret


class TranslationsMixin(object):
    ''' Adds support for nested translations in a serializer
        Generated field will default to the model's translation accessor.
    '''

    # Add the translations accessor to default serializer fields
    def get_default_field_names(self, *args):
        return (
            super(TranslationsMixin, self).get_default_field_names(*args)
            + [self.Meta.model._meta.translations_accessor]
        )

    def build_field(self, field_name, info, model_class, nested_depth):
        # Special handling for translations field so it is nested and not relational
        if field_name == model_class._meta.translations_accessor:
            # Create a nested serializer as a subclass of configured translations_serializer
            BaseSerializer = getattr(self.Meta, 'translations_serializer', NestedTranslationSerializer)
            BaseMeta = getattr(BaseSerializer, 'Meta', None)
            exclude = veto_fields + ('language_code',)
            if BaseMeta is not None:
                exclude += tuple(getattr(BaseMeta, 'exclude', ()))

            NestedMeta = type('Meta', (object,) if BaseMeta is None else (BaseMeta, object), {
                'model': model_class._meta.translations_model,
                'exclude': exclude,
                'depth': nested_depth,
                'list_serializer_class': TranslationListSerializer,
            })
            NestedSerializer = type('NestedSerializer', (BaseSerializer,), {'Meta': NestedMeta})

            kwargs = {'many': True}
            if isinstance(self, TranslatableModelMixin):
                kwargs['required'] = False
            return NestedSerializer, kwargs

        return super(TranslationsMixin, self).build_field(field_name, info, model_class, nested_depth)

    def to_internal_value(self, data):
        # Allow TranslationsMixin to be combined with TranslatableModelSerializer
        # This means we allow translated fields to be absent if translations are set

        if isinstance(self, TranslatableModelMixin):
            tmodel = self.Meta.model._meta.translations_model

            # Look for translated fields, and mark them read_only if translations is set
            if self.Meta.model._meta.translations_accessor in data:
                for name, field in self.fields.items():
                    source = field.source or field.field_name
                    if source in veto_fields:
                        continue # not a translated field
                    try:
                        tmodel._meta.get_field(source)
                    except FieldDoesNotExist:
                        continue # not a translated field
                    field.read_only = True

        return super(TranslationsMixin, self).to_internal_value(data)

    def create(self, data):
        accessor = self.Meta.model._meta.translations_accessor
        translations_data = data.pop(accessor, None)
        if translations_data:
            arbitrary = translations_data.popitem()
            data.update(arbitrary[1])
            data['language_code'] = arbitrary[0]
            instance = super(TranslationsMixin, self).create(data)

            for language, translation_data in translations_data.items():
                instance.translate(language)
                self.update_translation(instance, translation_data)
        else:
            instance = super(TranslationsMixin, self).create(data)
        return instance

    def update(self, instance, data):
        accessor = self.Meta.model._meta.translations_accessor
        translations_data = data.pop(accessor, None)
        if translations_data:
            arbitrary = translations_data.popitem()
            data.update(arbitrary[1])
            stashed = set_cached_translation(
                instance, load_translation(instance, arbitrary[0], enforce=True)
            )
            instance = super(TranslationsMixin, self).update(instance, data)

            for language, translation_data in translations_data.items():
                set_cached_translation(instance, load_translation(instance, language, enforce=True))
                self.update_translation(instance, translation_data)

            set_cached_translation(instance, stashed)
            qs = instance._meta.translations_model.objects
            (qs.filter(master=instance)
                .exclude(language_code__in=(arbitrary[0],)+tuple(translations_data.keys()))
                .delete())
        else:
            instance = super(TranslationsMixin, self).update(instance, data)
        return instance

    def update_translation(self, instance, data):
        if django.VERSION >= (1, 8):
            fields = set(field.name
                         for field in self.Meta.model._meta.translations_model._meta.get_fields()
                         if not field.is_relation or                    # regular fields are ok
                            field.one_to_one or                         # one to one is ok
                            field.many_to_one and field.related_model)  # many_to_one only if not generic
        else:
            fields = set(name
                         for name in self.Meta.model._meta.translations_model._meta.get_all_field_names())
        fields.intersection_update(data)
        vetoed = fields.intersection('id', 'master', 'master_id', 'language_code')
        if vetoed:
            raise KeyError('These fields are not allowed in data: ' % ', '.join(vetoed))

        for key, value in data.items():
            setattr(instance, key, value)
        instance.save(update_fields=fields)

#=============================================================================

class TranslatableModelMixin(object):
    ''' Adds support for translated fields on a serializer '''
    default_error_messages = {
        'enforce_violation': _l('Sending a language_code is invalid on serializers '
                                'that enforce a language'),
    }

    def __init__(self, *args, **kwargs):
        # We use an exception because None is a valid value for language
        try:
            self.language = kwargs.pop('language')
        except KeyError:
            pass
        super(TranslatableModelMixin, self).__init__(*args, **kwargs)

    def get_default_field_names(self, *args):
        # Add translated fields into default field names
        return (
            super(TranslatableModelMixin, self).get_default_field_names(*args)
            + list(field.name
                   for field in self.Meta.model._meta.translations_model._meta.fields
                   if field.serialize and not field.name in veto_fields)
        )

    def get_uniqueness_extra_kwargs(self, field_names, declared_fields, *args):
        # Default implementation chokes on translated fields, filter them out
        shared_fields = []
        for field_name in field_names:
            field = declared_fields.get(field_name)
            if field is not None:
                field_name = field.source or field_name
            if field_name not in veto_fields:
                try:
                    self.Meta.model._meta.translations_model._meta.get_field(field_name)
                except FieldDoesNotExist:
                    pass
                else:
                    continue
            shared_fields.append(field_name)

        return super(TranslatableModelMixin, self).get_uniqueness_extra_kwargs(shared_fields, declared_fields, *args)

    def build_field(self, field_name, info, model_class, nested_depth):
        # Special case the language code field - we handle it manually
        if field_name == 'language_code':
            field = model_class._meta.translations_model._meta.get_field(field_name)
            klass, kwargs = self.build_standard_field(field_name, field)
            kwargs['required'] = False
            return klass, kwargs

        # Try to find a translated field matching the description
        field = None
        if field_name not in veto_fields:
            try:
                field = model_class._meta.translations_model._meta.get_field(field_name)
            except FieldDoesNotExist:
                pass
        if field is not None:
            return self.build_standard_field(field_name, field)

        # Nothing unusual, let rest_framework do its stuff
        return super(TranslatableModelMixin, self).build_field(
                field_name, info, model_class, nested_depth
        )

    def to_representation(self, instance):
        'Switch language if we are in enforce mode'
        enforce = hasattr(self, 'language')
        language = getattr(self, 'language', None) or get_language()

        translation = load_translation(instance, language, enforce)
        set_cached_translation(instance, translation)

        return super(TranslatableModelMixin, self).to_representation(instance)

    def validate(self, data):
        data = super(TranslatableModelMixin, self).validate(data)
        if hasattr(self, 'language'):
            if 'language_code' in data:
                raise ValidationError(self.error_messages['enforce_violation'])
            data['language_code'] = self.language
        return data

    def update(self, instance, data):
        'Handle switching to correct translation before actual update'
        enforce = 'language_code' in data
        language = data.pop('language_code', None) or get_language()
        translation = load_translation(instance, language, enforce)
        set_cached_translation(instance, translation)

        return super(TranslatableModelMixin, self).update(instance, data)

#=============================================================================

class TranslatableModelSerializer(TranslatableModelMixin, serializers.ModelSerializer):
    pass

class HyperlinkedTranslatableModelSerializer(TranslatableModelMixin,
                                             serializers.HyperlinkedModelSerializer):
    pass



import django
from django.utils.functional import cached_property
from django.test.testcases import TestCase
from django.test.client import RequestFactory
from hvad.test_utils.context_managers import UserLoginContext
import warnings


def minimumDjangoVersion(*args):
    return (lambda x: x) if django.VERSION >= args else (lambda x: 'disabled')
def maximumDjangoVersion(*args):
    return (lambda x: x) if django.VERSION < args else (lambda x: 'disabled')


class _AssertThrowsWarningContext(object):
    def __init__(self, test_case, klass, number):
        self.test_case = test_case
        self.klass = klass
        self.number = number
        self.ctx = warnings.catch_warnings(record=True)

    def __enter__(self):
        self.warnings = self.ctx.__enter__()
        warnings.resetwarnings()
        warnings.simplefilter('always')

    def __exit__(self, type, value, traceback):
        self.test_case.assertEqual(
            len(self.warnings), self.number, "%d warnings thrown, %d expected" % (
                len(self.warnings), self.number
            )
        )
        for warning in self.warnings:
            self.test_case.assertTrue(issubclass(warning.category, self.klass),
                                      '%s warning thrown, %s expected' %
                                      (warning.category.__name__, self.klass.__name__))
        self.ctx.__exit__(type, value, traceback)


class HvadTestCase(TestCase):
    def setUp(self):

        if hasattr(self, 'create_fixtures'):
            self.create_fixtures()

    @cached_property
    def request_factory(self):
        return RequestFactory()

    def login_user_context(self, username):
        return UserLoginContext(self, username=username, password=username)

    def assertThrowsWarning(self, klass, number=1):
        return _AssertThrowsWarningContext(self, klass, number)

    def assertSavedObject(self, obj, language, **kwargs):
        'Checks the object was saved in given language with given attributes'
        self.assertEqual(language, kwargs.pop('language_code', language),
                         'Test error: mismatching language and language_code.')
        for key, value in kwargs.items():
            self.assertEqual(getattr(obj, key), value)
        self.assertEqual(obj.language_code, language)
        self.assertCountEqual(
            obj.__class__.objects.language(language).filter(**kwargs).values_list('pk', flat=True),
            [obj.pk]
        )

# method was renamed from assertItemsEqual in Python 3
if not hasattr(HvadTestCase, 'assertCountEqual'):
    HvadTestCase.assertCountEqual = HvadTestCase.assertItemsEqual



# -*- coding: utf-8 -*-
#
# Project 何 documentation build configuration file, created by
# sphinx-quickstart on Tue Feb 15 11:19:28 2011.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys, os, sphinx

readthedocs = os.environ.get('READTHEDOCS', None) == 'True'

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

sys.path.extend((
    os.path.abspath(os.path.join(os.path.dirname(__file__), '_ext')),
))

# -- General configuration -----------------------------------------------------

extensions = [
    'sphinx.ext.intersphinx',
    'djangodocs',
    'github'
]
intersphinx_mapping = {
    'python': ('http://docs.python.org/2.7', None),
    'django': ('http://readthedocs.org/docs/django/en/latest/', None),
}

templates_path = ['_templates']
source_suffix = '.rst'
exclude_patterns = ['_build']
master_doc = 'index'

# General information about the project.
project = u'django-hvad'
copyright = u'2011-2015, Kristian Øllegaard, Jonas Obrist & contributors'

version = '1.6'
release = '1.6.0'


# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'trac'

# -- Options for HTML output ---------------------------------------------------

if sphinx.version_info >= (1, 3):
    html_theme = 'default' if readthedocs else 'classic'
else:
    html_theme = 'default'
html_theme_options = {
    'externalrefs': True,
}

html_style = 'stylesheet.css'
html_static_path = ['_static']

html_use_smartypants = True
html_show_sourcelink = False

# Custom sidebar templates, maps document names to template names.
html_sidebars = {'**': ['globaltoc.html', 'relations.html', 'sourcelink.html', 'searchbox.html']}


# Output file base name for HTML help builder.
htmlhelp_basename = 'Projectdoc'


# -- Options for LaTeX output --------------------------------------------------

# The paper size ('letter' or 'a4').
#latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
#latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
  ('index', 'Project.tex', u'Django Hvad Documentation',
   u'Jonas Obrist & contributors', 'manual'),
]


# -- Options for manual page output --------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'project', u'Django Hvad Documentation',
     [u'Kristian Øllegaard, Jonas Obrist & contributors'], 1)
]

# -- Options for local extensions ----------------------------------------------

github_owner = 'KristianOellegaard'
github_repo = 'django-hvad'




from django.utils.translation import ugettext_lazy as _l
from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from rest_framework.settings import api_settings
from hvad.utils import get_cached_translation, set_cached_translation

#=============================================================================

class TranslationListSerializer(serializers.ListSerializer):
    'A custom serializer to output translations in a nice dict'
    many = True
    default_error_messages = {
        'not_a_dict': _l('Expected a dictionary of items, but got a {input_type}.'),
        'no_translation': _l('At least one translation must be provided.'),
    }

    def to_internal_value(self, data):
        if not isinstance(data, dict):
            message = self.error_messages['not_a_dict'].format(
                input_type=type(data).__name__
            )
            raise ValidationError({
                api_settings.NON_FIELD_ERRORS_KEY: [message]
            })
        if not data:
            message = self.error_messages['no_translation']
            raise ValidationError({
                api_settings.NON_FIELD_ERRORS_KEY: [message]
            })

        ret, errors = {}, {}
        for language, translation in data.items():
            try:
                validated = self.child.run_validation(translation)
            except ValidationError as exc:
                errors[language] = exc.detail
            else:
                ret[language] = validated
                errors[language] = {}
        if any(errors.values()):
            raise ValidationError(errors)
        return ret

    def get_attribute(self, instance):
        ''' Override get_attribute so it returns the whole translatable model and not just translations '''
        return instance

    def to_representation(self, instance):
        ''' Combine each translation in turn so the serializer has a full object '''
        result = {}
        stashed = get_cached_translation(instance)
        for translation in getattr(instance, self.source).all():
            set_cached_translation(instance, translation)
            result[translation.language_code] = self.child.to_representation(instance)
        set_cached_translation(instance, stashed)
        return result

    def save(self, *args, **kwargs): #pragma: no cover
        raise NotImplementedError('TranslationList must be nested')
    @property
    def data(self): #pragma: no cover
        raise NotImplementedError('TranslationList must be nested')
    @property
    def errors(self): #pragma: no cover
        raise NotImplementedError('TranslationList must be nested')



# Sphinx helper for github issue references
from docutils import nodes
from docutils.parsers.rst.roles import set_classes

ISSUE_URL = 'https://github.com/{owner}/{repo}/issues/{num}'

def github_issue_role(role, rawtext, text, lineno, inliner, options={}, content=[]):
    try:
        issuenum = int(text)
        if issuenum <= 0:
            raise ValueError()
    except ValueError:
        msg = inliner.reporter.error(
            'Issuse number must be a strictly positive integer. Number "%s" '
            'is invalid.' % text, line=lineno)
        prb = inliner.problematic(rawtext, rawtext, msg)
        return [prb], [msg]

    ref = ISSUE_URL.format(
        owner = inliner.document.settings.env.config['github_owner'],
        repo = inliner.document.settings.env.config['github_repo'],
        num = issuenum,
    )
    set_classes(options)
    node = nodes.reference(rawtext, '#%d' % issuenum, refuri=ref, **options)
    return [node], []

def setup(app):
    app.add_config_value('github_owner', None, 'html')
    app.add_config_value('github_repo', None, 'html')
    app.add_role('issue', github_issue_role)



import django
if django.VERSION >= (1, 7):
    from django.core import checks
from django.core.exceptions import ImproperlyConfigured
from django.conf import settings
from django.db import models
from django.db.models.base import ModelBase
from django.db.models.fields import FieldDoesNotExist
from django.db.models.manager import Manager
from django.db.models.signals import post_save, class_prepared
from django.utils.translation import get_language
from hvad.descriptors import LanguageCodeAttribute, TranslatedAttribute
from hvad.manager import TranslationManager, TranslationsModelManager
from hvad.utils import (get_cached_translation, set_cached_translation,
                        SmartGetFieldByName, SmartGetField, settings_updater)
from hvad.compat import MethodType
from itertools import chain
import sys
import warnings

#===============================================================================

# Global settings, wrapped so they react to SettingsOverride
@settings_updater
def update_settings(*args, **kwargs):
    global FALLBACK_LANGUAGES, TABLE_NAME_SEPARATOR
    FALLBACK_LANGUAGES = tuple( code for code, name in settings.LANGUAGES )
    TABLE_NAME_SEPARATOR = getattr(settings, 'HVAD_TABLE_NAME_SEPARATOR', '_')

#===============================================================================

class TranslatedFields(object):
    """ Wrapper class to define translated fields on a model. """

    def __init__(self, meta=None, base_class=None, **fields):
        self.meta = meta or {}
        self.base_class = base_class
        self.fields = fields

    @staticmethod
    def _split_together(constraints, fields, meta, name):
        sconst, tconst = [], []
        if name in meta:
            # raise in 1.4, remove in 1.6
            warnings.warn('Passing \'%s\' to TranslatedFields is deprecated. Please use '
                        'Please Meta.%s instead.' % (name, name), DeprecationWarning)
            tconst.extend(meta[name])

        for constraint in constraints:
            if all(item in fields for item in constraint):
                tconst.append(constraint)
            elif not any(item in fields for item in constraint):
                sconst.append(constraint)
            else:
                raise ImproperlyConfigured(
                    'Constraints in Meta.%s cannot mix translated and '
                    'untranslated fields, such as %r.' % (name, constraint))
        return sconst, tconst

    def contribute_to_class(self, model, name):
        if model._meta.order_with_respect_to in self.fields:
            raise ValueError(
                'Using a translated fields in %s.Meta.order_with_respect_to is ambiguous '
                'and hvad does not support it.' %
                model._meta.model_name
            )
        if hasattr(model._meta, 'translations_model'):
            raise ImproperlyConfigured(
                "A TranslatableModel can only define one set of "
                "TranslatedFields, %r defines more than one." % model
            )
        translations_model = self.create_translations_model(model, name)
        model._meta.translations_model = translations_model
        if not model._meta.abstract:
            self.contribute_translations(model, translations_model, name)

    def create_translations_model(self, model, related_name):
        """ Create the translations model for a shared model.
            model -- the model class to create translations for
            related_name -- the related name for the reverse FK from the translations model.
        """
        model_name = '%sTranslation' % model.__name__
        translation_bases, translation_base_fields = self._scan_model_bases(model)

        attrs = self.fields.copy()
        attrs.update({
            'Meta': self._build_meta_class(
                model, translation_base_fields.union(self.fields).union(('language_code',))
            ),
            '__module__': model.__module__,
        })

        if not model._meta.abstract:
            attrs.update({
                # If this class is abstract, we must not contribute management fields
                'objects': TranslationsModelManager(),
                'language_code': models.CharField(max_length=15, db_index=True),
                # Nullable so we can prevent cascade deletion
                'master': models.ForeignKey(model, related_name=related_name, editable=False,
                                            on_delete=models.CASCADE),
            })

        # Create the new model
        if self.base_class:
            translation_bases.insert(0, self.base_class)
        translations_model = ModelBase(model_name, tuple(translation_bases), attrs)
        translations_model._meta.shared_model = model
        if not model._meta.abstract:
            # Abstract models do not have a DNE class
            bases = (model.DoesNotExist, translations_model.DoesNotExist,)
            translations_model.DoesNotExist = type('DoesNotExist', bases, {})

        # Register it as a global in the shared model's module.
        # This is needed so that Translation model instances, and objects which
        # refer to them, can be properly pickled and unpickled. The Django session
        # and caching frameworks, in particular, depend on this behaviour.
        setattr(sys.modules[model.__module__], model_name, translations_model)
        return translations_model

    def _scan_model_bases(self, model):
        """ Scan the model class' bases, collecting translated fields """
        bases, fields = list(), set()
        scan_bases = list(reversed(model.__bases__))
        while scan_bases:
            base = scan_bases.pop()
            if base is TranslatableModel or not issubclass(base, TranslatableModel):
                continue
            if not base._meta.abstract:
                raise TypeError(
                    'Multi-table inheritance of translatable models is not supported. '
                    'Concrete model %s is not a valid base model for %s.' %
                    (base._meta.model_name, model._meta.model_name)
                )
            # The base may have translations model, then just inherit that
            if hasattr(base._meta, 'translations_model'):
                bases.append(base._meta.translations_model)
                fields.update(field.name for field in base._meta.translations_model._meta.fields)
            else:
                # But it may not, and simply inherit other abstract bases, scan them
                scan_bases.extend(reversed(base.__bases__))
        bases.append(BaseTranslationModel)
        return bases, fields

    def _build_meta_class(self, model, tfields):
        """ Create the Meta class for the translation model
            model -- the shared model
            tfields -- the list of names of all fields, direct and inherited
        """
        abstract = model._meta.abstract
        meta = self.meta.copy()
        meta.update({
            'abstract': abstract,
            'db_tablespace': model._meta.db_tablespace,
            'managed': model._meta.managed,
            'app_label': model._meta.app_label,
            'db_table': meta.get('db_table',
                                 '%s%stranslation' % (model._meta.db_table, TABLE_NAME_SEPARATOR)),
        })
        if django.VERSION >= (1, 7):
            meta['default_permissions'] = ()

        # Split fields in Meta.unique_together
        sconst, tconst = self._split_together(
            model._meta.unique_together, tfields, meta, 'unique_together'
        )
        model._meta.unique_together = tuple(sconst)
        if django.VERSION >= (1, 7):
            model._meta.original_attrs['unique_together'] = tuple(sconst)
        meta['unique_together'] = tuple(tconst)
        if not abstract:
            meta['unique_together'] += (('language_code', 'master'),)

        # Split fields in Meta.index_together
        sconst, tconst = self._split_together(
            model._meta.index_together, tfields, meta, 'index_together'
        )
        model._meta.index_together = tuple(sconst)
        if django.VERSION >= (1, 7):
            model._meta.original_attrs['index_together'] = tuple(sconst)
        meta['index_together'] = tuple(tconst)

        return type('Meta', (object,), meta)

    def contribute_translations(self, model, translations_model, related_name):
        """ Contribute translations model to the Meta class and set descriptors """

        model._meta.translations_accessor = related_name
        model._meta.translations_cache = '%s_cache' % related_name

        # Set descriptors
        ignore_fields = ('pk', 'master', 'master_id', translations_model._meta.pk.name)
        for field in translations_model._meta.fields:
            if field.name in ignore_fields:
                continue
            if field.name == 'language_code':
                attr = LanguageCodeAttribute(model._meta)
            else:
                attr = TranslatedAttribute(model._meta, field.name)
                attname = field.get_attname()
                if attname and attname != field.name:
                    setattr(model, attname, TranslatedAttribute(model._meta, attname))
            setattr(model, field.name, attr)

#===============================================================================

class BaseTranslationModel(models.Model):
    """
    Needed for detection of translation models. Due to the way dynamic classes
    are created, we cannot put the 'language_code' field on here.
    """
    def _get_unique_checks(self, exclude=None):
        # Due to the way translations are handled, checking for unicity of
        # the ('language_code', 'master') constraint is useless. We filter it out
        # here so as to avoid a useless query
        unique_checks, date_checks = super(BaseTranslationModel, self)._get_unique_checks(exclude=exclude)
        unique_checks = [check for check in unique_checks
                         if check != (self.__class__, ('language_code', 'master'))]
        return unique_checks, date_checks

    class Meta:
        abstract = True


class NoTranslation(object):
    pass


class TranslatableModel(models.Model):
    """
    Base model for all models supporting translated fields (via TranslatedFields).
    """
    # change the default manager to the translation manager
    objects = TranslationManager()
    _plain_manager = models.Manager()

    class Meta:
        abstract = True
        if django.VERSION >= (1, 10):
            base_manager_name = '_plain_manager'

    def __init__(self, *args, **kwargs):
        # Split arguments into shared/translatd
        veto_names = ('pk', 'master', 'master_id', self._meta.translations_model._meta.pk.name)
        skwargs, tkwargs = {}, {}
        for key, value in kwargs.items():
            if key in self._translated_field_names and not key in veto_names:
                tkwargs[key] = value
            else:
                skwargs[key] = value

        super(TranslatableModel, self).__init__(*args, **skwargs)

        # Create a translation if there are translated fields
        if tkwargs:
            tkwargs['language_code'] = tkwargs.get('language_code') or get_language()
            set_cached_translation(self, self._meta.translations_model(**tkwargs))

    def save(self, *args, **skwargs):
        translation_model = self._meta.translations_model
        translation = get_cached_translation(self)
        tkwargs = skwargs.copy()

        # split update_fields in shared/translated fields
        update_fields = skwargs.get('update_fields')
        if update_fields is not None:
            supdate, tupdate = [], []
            for name in update_fields:
                if name in self._translated_field_names and not name in ('id', 'master_id', 'master'):
                    tupdate.append(name)
                else:
                    supdate.append(name)
            skwargs['update_fields'], tkwargs['update_fields'] = supdate, tupdate

        # save share and translated model in a single transaction
        if update_fields is None or skwargs['update_fields']:
            super(TranslatableModel, self).save(*args, **skwargs)
        if (update_fields is None or tkwargs['update_fields']) and translation is not None:
            if translation.pk is None and update_fields:
                del tkwargs['update_fields'] # allow new translations
            translation.master = self
            translation.save(*args, **tkwargs)
    save.alters_data = True

    def translate(self, language_code):
        ''' Create a new translation for current instance.
            Does NOT check if the translation already exists!
        '''
        set_cached_translation(
            self,
            self._meta.translations_model(language_code=language_code)
        )
        return self
    translate.alters_data = True

    def safe_translation_getter(self, name, default=None):
        cache = get_cached_translation(self)
        if cache is None:
            return default
        return getattr(cache, name, default)

    def lazy_translation_getter(self, name, default=None):
        """
        Lazy translation getter that fetches translations from DB in case the instance is currently untranslated and
        saves the translation instance in the translation cache
        """
        stuff = self.safe_translation_getter(name, NoTranslation)
        if stuff is not NoTranslation:
            return stuff

        # get all translations
        translations = getattr(self, self._meta.translations_accessor).all()

        # if no translation exists, bail out now
        if len(translations) == 0:
            return default

        # organize translations into a nice dict
        translation_dict = dict((t.language_code, t) for t in translations)

        # see if we have the right language, or any language in fallbacks
        for code in (get_language(), settings.LANGUAGE_CODE) + FALLBACK_LANGUAGES:
            try:
                translation = translation_dict[code]
            except KeyError:
                continue
            break
        else:
            # none of the fallbacks was found, pick an arbitrary translation
            translation = translation_dict.popitem()[1]

        set_cached_translation(self, translation)
        return getattr(translation, name, default)

    def get_available_languages(self):
        """ Get a list of all available language_code in db. """
        qs = getattr(self, self._meta.translations_accessor).all()
        if qs._result_cache is not None:
            return [obj.language_code for obj in qs]
        return qs.values_list('language_code', flat=True)

    #===========================================================================
    # Validation
    #===========================================================================

    def clean_fields(self, exclude=None):
        super(TranslatableModel, self).clean_fields(exclude=exclude)
        translation = get_cached_translation(self)
        if translation is not None:
            translation.clean_fields(exclude=exclude + ['id', 'master', 'master_id', 'language_code'])

    def validate_unique(self, exclude=None):
        super(TranslatableModel, self).validate_unique(exclude=exclude)
        translation = get_cached_translation(self)
        if translation is not None:
            translation.validate_unique(exclude=exclude)

    #===========================================================================
    # Checks - require Django 1.7 or newer
    #===========================================================================

    if django.VERSION >= (1, 7):
        @classmethod
        def check(cls, **kwargs):
            errors = super(TranslatableModel, cls).check(**kwargs)
            errors.extend(cls._check_shared_translated_clash())
            return errors

        @classmethod
        def _check_shared_translated_clash(cls):
            fields = set(chain.from_iterable(
                (f.name, f.attname)
                for f in cls._meta.fields
            ))
            tfields = set(chain.from_iterable(
                (f.name, f.attname)
                for f in cls._meta.translations_model._meta.fields
                if f.name not in ('id', 'master')
            ))
            return [checks.Error("translated field '%s' clashes with untranslated field." % field,
                                hint=None, obj=cls, id='hvad.models.E01')
                    for field in tfields.intersection(fields)]

        @classmethod
        def _check_local_fields(cls, fields, option):
            """ Remove fields we recognize as translated fields from tests """
            to_check = []
            for field in fields:
                try:
                    cls._meta.translations_model._meta.get_field(field)
                except FieldDoesNotExist:
                    to_check.append(field)
            return super(TranslatableModel, cls)._check_local_fields(to_check, option)

        @classmethod
        def _check_ordering(cls):
            if not cls._meta.ordering:
                return []

            if not isinstance(cls._meta.ordering, (list, tuple)):
                return [checks.Error("'ordering' must be a tuple or list.",
                                    hint=None, obj=cls, id='models.E014')]

            fields = [f for f in cls._meta.ordering if f != '?']
            fields = [f[1:] if f.startswith('-') else f for f in fields]
            fields = set(f for f in fields if f not in ('_order', 'pk') and '__' not in f)

            valid_fields = set(chain.from_iterable(
                (f.name, f.attname)
                for f in cls._meta.fields
            ))
            valid_tfields = set(chain.from_iterable(
                (f.name, f.attname)
                for f in cls._meta.translations_model._meta.fields
                if f.name not in ('master', 'language_code')
            ))

            return [checks.Error("'ordering' refers to the non-existent field '%s' --hvad." % field,
                                hint=None, obj=cls, id='models.E015')
                    for field in fields - valid_fields - valid_tfields]

    #===========================================================================
    # Internals
    #===========================================================================
    
    @property
    def _translated_field_names(self):
        if getattr(self, '_translated_field_names_cache', None) is None:
            opts = self._meta.translations_model._meta
            result = set()

            if django.VERSION >= (1, 8):
                for field in opts.get_fields():
                    result.add(field.name)
                    if hasattr(field, 'attname'):
                        result.add(field.attname)
            else:
                result = set(opts.get_all_field_names())
                for name in tuple(result):
                    try:
                        attname = opts.get_field(name).get_attname()
                    except (FieldDoesNotExist, AttributeError):
                        continue
                    if attname:
                        result.add(attname)

            self._translated_field_names_cache = tuple(result)
        return self._translated_field_names_cache

#=============================================================================

def prepare_translatable_model(sender, **kwargs):
    model = sender
    if not issubclass(model, TranslatableModel) or model._meta.abstract:
        return
    if not isinstance(model._default_manager, TranslationManager):
        raise ImproperlyConfigured(
            "The default manager on a TranslatableModel must be a "
            "TranslationManager instance or an instance of a subclass of "
            "TranslationManager, the default manager of %r is not." % model)

    if model._meta.proxy:
        model._meta.translations_accessor = model._meta.concrete_model._meta.translations_accessor
        model._meta.translations_model = model._meta.concrete_model._meta.translations_model
        model._meta.translations_cache = model._meta.concrete_model._meta.translations_cache

    if not hasattr(model._meta, 'translations_model'):
        raise ImproperlyConfigured(
            "No TranslatedFields found on %r, subclasses of "
            "TranslatableModel must define TranslatedFields." % model
        )

    #### Now we have to work ####

    # Ensure _base_manager cannot be TranslationManager despite use_for_related_fields
    # 1- it is useless unless default_class is overriden
    # 2- in that case, _base_manager is used for saving objects and must not be
    #    translation aware.
    base_mgr = getattr(model, '_base_manager', None)
    if base_mgr is None or isinstance(base_mgr, TranslationManager):
        assert django.VERSION < (1, 10)
        model.add_to_class('_base_manager', Manager())

    # Replace get_field_by_name with one that warns for common mistakes
    if django.VERSION < (1, 9) and not isinstance(model._meta.get_field_by_name, SmartGetFieldByName):
        model._meta.get_field_by_name = MethodType(
            SmartGetFieldByName(model._meta.get_field_by_name),
            model._meta
        )
    if not isinstance(model._meta.get_field, SmartGetField):
        model._meta.get_field = MethodType(
            SmartGetField(model._meta.get_field),
            model._meta
        )

class_prepared.connect(prepare_translatable_model)



from django.views.generic.detail import SingleObjectMixin, SingleObjectTemplateResponseMixin
from django.views.generic.edit import ModelFormMixin, ProcessFormView, BaseDeleteView
from django.utils.translation import get_language
from hvad.forms import translatable_modelform_factory
from hvad.utils import collect_context_modifiers
import warnings

class _TransitionObjectMixin(SingleObjectMixin):
    # Remove in 1.5
    def get_object(self, queryset=None):
        assert not callable(getattr(self, '_get_object', None)), (
            'Method \'_get_object()\' was removed. Please update view %s to use '
            '\'get_object()\' instead.' % self.__class__.__name__)

        assert not callable(getattr(self, 'filter_kwargs', None)), (
            'Method \'filter_kwargs()\' was removed. Please update view %s to use '
            '\'get_queryset()\' or \'get_object()\'.' % self.__class__.__name__)

        assert not (self.pk_url_kwarg == 'pk' and 'object_id' in self.kwargs and 'pk' not in self.kwargs), (
            'Default view argument for pk has changed from \'object_id\' '
            'to \'pk\'. Please update view %s.' % self.__class__.__name__)

        return super(_TransitionObjectMixin, self).get_object(queryset)


class TranslatableModelFormMixin(ModelFormMixin, _TransitionObjectMixin):
    ''' ModelFormMixin that works with an TranslatableModelForm in **enforce** mode '''
    query_language_key = 'language'

    def get_language(self):
        # Remove in 1.5
        assert not callable(getattr(self, '_language', None)), (
            'Method \'_language\' has been renamed to \'get_language()\'. '
            'Please update view %s.' % self.__class__.__name__)
        return self.request.GET.get(self.query_language_key) or get_language()

    def get_form_class(self):
        if self.model is not None:
            model = self.model
        elif getattr(self, 'object', None) is not None:
            model = self.object.__class__
        else:
            qs = self.get_queryset()
            model = getattr(qs, 'shared_model', qs.model)

        kwargs = {}
        if self.form_class is not None:
            kwargs['form'] = self.form_class
        return translatable_modelform_factory(self.get_language(), model, **kwargs)

    def get_context_data(self, **kwargs):
        # Deprecation warning is triggered inside collect_context_modifiers
        # remove this in 1.5
        context = super(TranslatableModelFormMixin, self).get_context_data(**kwargs)
        context.update(collect_context_modifiers(self, extra_kwargs=kwargs))
        return context

#=============================================================================

class TranslatableBaseCreateView(TranslatableModelFormMixin, ProcessFormView):
    def get(self, request, *args, **kwargs):
        self.object = None
        return super(TranslatableBaseCreateView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = None
        return super(TranslatableBaseCreateView, self).post(request, *args, **kwargs)

class TranslatableCreateView(SingleObjectTemplateResponseMixin, TranslatableBaseCreateView):
    template_name_suffix = '_form'

#-------------------------------------------------------------------------

class TranslatableBaseUpdateView(TranslatableModelFormMixin, ProcessFormView):
    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(TranslatableBaseUpdateView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(TranslatableBaseUpdateView, self).post(request, *args, **kwargs)

class TranslatableUpdateView(SingleObjectTemplateResponseMixin, TranslatableBaseUpdateView):
    template_name_suffix = '_form'

#-------------------------------------------------------------------------

class TranslatableBaseDeleteView(BaseDeleteView, _TransitionObjectMixin):
    pass

class TranslatableDeleteView(SingleObjectTemplateResponseMixin, TranslatableBaseDeleteView):
    template_name_suffix = '_confirm_delete'

#=============================================================================
#=============================================================================
#=============================================================================

from django.views.generic.edit import UpdateView
from hvad.admin import TranslatableModelAdminMixin
from hvad.forms import TranslatableModelForm

class TranslatableBaseView(UpdateView, TranslatableModelAdminMixin): #pragma: no cover
    # Remove in 1.5
    form_class = TranslatableModelForm

    def __init__(self, *args, **kwargs):
        raise AssertionError(
            'TranslatableBaseView has been removed. Please update view %s to use '
            'new Django-compliant view instead.' % self.__class__.__name__
        )



from collections import defaultdict
import django
from django.conf import settings
from django.core.exceptions import FieldError
from django.db import connections, models, transaction, IntegrityError
if django.VERSION >= (1, 9):
    from django.db.models.query import QuerySet
elif django.VERSION >= (1, 8):
    from django.db.models.query import QuerySet, ValuesQuerySet
else:
    from django.db.models.query import QuerySet, ValuesQuerySet, DateQuerySet, DateTimeQuerySet
if django.VERSION >= (1, 8):
    from django.db.models.sql.datastructures import Join, LOUTER
from django.db.models.sql.constants import GET_ITERATOR_CHUNK_SIZE as CHUNK_SIZE
from django.db.models import F, Q
from django.utils.translation import get_language
from hvad.compat import string_types
from hvad.query import (query_terms, q_children, expression_nodes, where_node_children,
                        add_alias_constraints)
from hvad.utils import combine, minimumDjangoVersion, settings_updater
from copy import deepcopy
import logging
import sys
import warnings

#===============================================================================

# Logging-related globals
_logger = logging.getLogger(__name__)

# Global settings, wrapped so they react to SettingsOverride
@settings_updater
def update_settings(*args, **kwargs):
    global FALLBACK_LANGUAGES, LEGACY_FALLBACKS
    FALLBACK_LANGUAGES = tuple(code for code, name in settings.LANGUAGES)
    LEGACY_FALLBACKS = bool(getattr(settings, 'HVAD_LEGACY_FALLBACKS', False))

#===============================================================================

class FieldTranslator(object):
    """
    Translates *shared* field names from '<shared_field>' to
    'master__<shared_field>' and caches those names.
    """
    def __init__(self, manager):
        self._manager = manager
        if django.VERSION >= (1, 8):
            fields = set()
            for field in manager.shared_model._meta.get_fields():
                fields.add(field.name)
                if hasattr(field, 'attname'):
                    fields.add(field.attname)
            fields.add('pk')
            self._shared_fields = tuple(fields)
        else:
            self._shared_fields = tuple(manager.shared_model._meta.get_all_field_names()) + ('pk',)
        self._cache = dict()
        super(FieldTranslator, self).__init__()

    def __call__(self, key):
        try:
            ret = self._cache[key]
        except KeyError:
            ret = self._build(key)
            self._cache[key] = ret
        return ret

    def _build(self, key):
        """
        Checks if the selected field is a shared field
        and in that case, prefixes it with master___
        It also handles - and ? in case its called by
        order_by()
        """
        if key == "?":
            return key
        if key.startswith("-"):
            prefix, key = "-", key[1:]
        else:
            prefix = ""
        if key.startswith(self._shared_fields):
            return '%smaster__%s' % (prefix, key)
        else:
            return '%s%s' % (prefix, key)

#===============================================================================

if django.VERSION >= (1, 9):
    from django.db.models.query import ValuesIterable, ValuesListIterable, FlatValuesListIterable
    class TranslatedValuesIterable(ValuesIterable):
        def __iter__(self):
            queryset = self.queryset
            for row in super(TranslatedValuesIterable, self).__iter__():
                yield queryset._reverse_translate_fieldnames_dict(row)

    iterable_overrides = {
        ValuesIterable: TranslatedValuesIterable,
        ValuesListIterable: ValuesListIterable,
        FlatValuesListIterable: FlatValuesListIterable,
    }

else:
    class ValuesMixin(object):
        _skip_master_select = True

        def iterator(self):
            qs = self._clone()._add_language_filter()
            for row in super(ValuesMixin, qs).iterator():
                if isinstance(row, dict):
                    yield qs._reverse_translate_fieldnames_dict(row)
                else:
                    yield row

    class SkipMasterSelectMixin(object):
        _skip_master_select = True

#===============================================================================

class ForcedUniqueFields(object):
    """ Context manager that forces a set of fields to be unique while active """
    def __init__(self, fields):
        self.fields = fields

    def __enter__(self):
        for field in self.fields:
            field._unique = True

    def __exit__(self, *args):
        for field in self.fields:
            field._unique = False

#===============================================================================
# Field for language joins
#===============================================================================

class RawConstraint(object):
    def __init__(self, sql, aliases):
        self.sql = sql
        self.aliases = aliases

    def as_sql(self, compiler, connection):
        aliases = tuple(compiler.quote_name_unless_alias(alias) for alias in self.aliases)
        return (self.sql % aliases, [])

class BetterTranslationsField(object):
    def __init__(self, translation_fallbacks, master):
        # Filter out duplicates, while preserving order
        self._fallbacks = []
        self._master = master
        seen = set()
        for lang in translation_fallbacks:
            if lang not in seen:
                seen.add(lang)
                self._fallbacks.append(lang)

    def get_extra_restriction(self, where_class, alias, related_alias):
        langcase = ('(CASE %s.language_code ' +
                    ' '.join('WHEN \'%s\' THEN %d' % (lang, i)
                             for i, lang in enumerate(self._fallbacks)) +
                    ' ELSE %d END)' % len(self._fallbacks))
        return RawConstraint(
            sql=' '.join((langcase, '<', langcase, 'OR ('
                          '%s.language_code = %s.language_code AND '
                          '%s.id < %s.id)')),
            aliases=(alias, related_alias,
                     alias, related_alias,
                     alias, related_alias)
        )

    @minimumDjangoVersion(1, 8)
    def get_joining_columns(self):
        return ((self._master, self._master), )

#===============================================================================
# TranslationQueryset
#===============================================================================

class TranslationQueryset(QuerySet):
    """
    This is where things happen.
    To fully understand this project, you have to understand this class.
    Go through each method individually, maybe start with 'get', 'create' and
    'iterator'.

    IMPORTANT: the `model` attribute on this class is the *translated* Model,
    despite this being used as the queryset for the *shared* Model!
    """
    override_classes = {}
    if django.VERSION < (1, 9):
        override_classes[ValuesQuerySet] = ValuesMixin
    if django.VERSION < (1, 8):
        override_classes[DateQuerySet] = SkipMasterSelectMixin
        override_classes[DateTimeQuerySet] = SkipMasterSelectMixin
    _skip_master_select = False

    def __init__(self, *args, **kwargs):
        # model can be either first positional, or a named arg
        if len(args) >= 1:
            model, args = args[0], args[1:]
        else:
            model = kwargs.pop('model', None)

        if model is not None:  # check the given model is correct
            if hasattr(model._meta, 'translations_model'):
                # normal creation gets a shared model that we must flip around
                model, self.shared_model = model._meta.translations_model, model
            elif not hasattr(model._meta, 'shared_model'):
                raise TypeError('TranslationQueryset only works on translatable models')

        self._local_field_names = None
        self._field_translator = None
        self._language_code = None
        self._language_fallbacks = None
        self._raw_select_related = []
        self._forced_unique_fields = []  # Used for select_related
        self._language_filter_tag = False
        self._hvad_switch_fields = ()
        super(TranslationQueryset, self).__init__(model, *args, **kwargs)

    #===========================================================================
    # Helpers and properties (INTERNAL!)
    #===========================================================================

    def _clone(self, klass=None, setup=False, **kwargs):
        """ Creates a clone of this queryset - Django equivalent of copy()
        This method keeps all defining attributes and drops data caches
        """
        kwargs.update({
            'shared_model': self.shared_model,
            '_local_field_names': self._local_field_names,
            '_field_translator': self._field_translator,
            '_language_code': self._language_code,
            '_language_fallbacks': self._language_fallbacks,
            '_raw_select_related': self._raw_select_related,
            '_forced_unique_fields': list(self._forced_unique_fields),
            '_language_filter_tag': getattr(self, '_language_filter_tag', False),
            '_hvad_switch_fields': self._hvad_switch_fields,
        })
        if django.VERSION < (1, 9):
            kwargs.update({
                'klass': None if klass is None else self._get_class(klass),
                'setup': setup,
            })
        return super(TranslationQueryset, self)._clone(**kwargs)

    @property
    def field_translator(self):
        if self._field_translator is None:
            self._field_translator = FieldTranslator(self)
        return self._field_translator

    @property
    def shared_local_field_names(self):
        if self._local_field_names is None:
            if django.VERSION >= (1, 8):
                fields = set()
                for field in self.shared_model._meta.get_fields():
                    fields.add(field.name)
                    if hasattr(field, 'attname'):
                        fields.add(field.attname)
                self._local_field_names = tuple(fields)
            else:
                self._local_field_names = self.shared_model._meta.get_all_field_names()
        return self._local_field_names

    def _translate_args_kwargs(self, *args, **kwargs):
        # Translate args (Q objects) from '<shared_field>' to
        # 'master__<shared_field>' where necessary.
        newargs = deepcopy(args)
        for q in newargs:
            for child, children, index in q_children(q):
                children[index] = (self.field_translator(child[0]), child[1])
        # Translated kwargs from '<shared_field>' to 'master__<shared_field>'
        # where necessary.
        newkwargs = dict((self.field_translator(key), value)
                         for key, value in kwargs.items())
        return newargs, newkwargs

    def _translate_expression(self, expr):
        if isinstance(expr, string_types):
            return self.field_translator(expr)
        for node in expression_nodes(expr):
            if isinstance(node, F):
                node.name = self.field_translator(node.name)
            elif hasattr(node, 'lookup'):
                node.lookup = self.field_translator(node.lookup)
        return expr

    def _translate_fieldnames(self, fieldnames):
        return [self.field_translator(name) for name in fieldnames]

    def _reverse_translate_fieldnames_dict(self, fieldname_dict):
        """
        Helper function to make sure the user doesnt get "bothered"
        with the construction of shared/translated model

        Translates e.g.
        {'master__number_avg': 10} to {'number__avg': 10}

        """
        prefix = 'master__'
        return dict(
            (key[len(prefix):] if key.startswith(prefix) else key, value)
            for key, value in fieldname_dict.items()
        )

    def _split_kwargs(self, **kwargs):
        """
        Split kwargs into shared and translated fields
        """
        shared = {}
        translated = {}
        for key, value in kwargs.items():
            if key in self.shared_local_field_names:
                shared[key] = value
            else:
                translated[key] = value
        return shared, translated

    def _get_class(self, klass):
        for key, value in self.override_classes.items():
            if issubclass(klass, key):
                return type(value.__name__, (value, klass, TranslationQueryset,), {})
        else: # pragma: no cover
            return klass

    def _get_shared_queryset(self):
        qs = super(TranslationQueryset, self)._clone()
        qs.__class__ = QuerySet
        accessor = self.shared_model._meta.translations_accessor
        # update using the real manager
        return QuerySet(self.shared_model, using=self.db).filter(**{'%s__in' % accessor: qs})

    def _add_select_related(self, language_code):
        fields = self._raw_select_related
        related_queries = []
        language_filters = []
        force_unique_fields = []
        if not self._skip_master_select and getattr(self, '_fields', None) is None:
            related_queries.append('master')

        for query_key in fields:
            newbits = []
            for term in query_terms(self.shared_model, query_key):

                # Translate term
                if term.depth == 0 and not term.translated:
                    # on initial depth we must key to shared model
                    newbits.append('master__%s' % term.term)
                elif term.depth > 0 and term.translated:
                    # on deeper levels we must key to translations model
                    # this will work because translations will be seen as _unique
                    # at query time
                    newbits.append('%s__%s' % (term.model._meta.translations_accessor, term.term))
                else:
                    newbits.append(term.term)

                # Some helpful messages for common mistakes
                if term.many:
                    raise FieldError('Cannot select_related: %s can be multiple objects. '
                                     'Use prefetch_related instead.' % query_key)
                if term.target is None:
                    raise FieldError('Cannot select_related: %s is a regular field' % query_key)
                if hasattr(term.field.rel, 'through'):
                    raise FieldError('Cannot select_related: %s can be multiple objects. '
                                     'Use prefetch_related instead.' % query_key)

                # If target is a translated model, select its translations
                target_translations = getattr(term.target._meta, 'translations_accessor', None)
                if target_translations is not None:
                    # Add the model
                    target_query = '__'.join(newbits)
                    related_queries.append('%s__%s' % (target_query, target_translations))

                    # Add a language filter for the translation
                    language_filters.append('%s__%s__language_code' % (
                        target_query,
                        target_translations,
                    ))

                    # Remember to mark the field unique so JOIN is generated
                    # and row decoder gets cached items
                    if django.VERSION >= (1, 9):
                        target_transfield = getattr(term.target, target_translations).field
                    else:
                        target_transfield = getattr(term.target, target_translations).related.field
                    force_unique_fields.append(target_transfield)

            related_queries.append('__'.join(newbits))

        # Apply results to query
        self.query.add_select_related(related_queries)
        for language_filter in language_filters:
            self.query.add_q(Q(**{language_filter: language_code}) |
                             Q(**{language_filter: None}))

        self._forced_unique_fields = force_unique_fields

    def _add_language_filter(self):
        if self._language_filter_tag: # pragma: no cover
            raise RuntimeError('Queryset is already tagged. This is a bug in hvad')
        self._language_filter_tag = True

        if self._language_code == 'all':
            self._add_select_related(F('language_code'))

        elif self._language_fallbacks:
            if self._raw_select_related:
                raise NotImplementedError('Using select_related along with '
                                          'fallbacks() is not supported')
            languages = tuple(get_language() if lang is None else lang
                              for lang in (self._language_code,) + self._language_fallbacks)

            if django.VERSION >= (1, 8):
                masteratt = self.model._meta.get_field('master').attname
                alias = self.query.join(Join(
                    self.model._meta.db_table,
                    self.query.get_initial_alias(),
                    None,
                    LOUTER,
                    BetterTranslationsField(languages, master=masteratt),
                    True
                ))
            else:
                masteratt = self.model._meta.get_field('master').attname
                nullable = ({'nullable': True} if django.VERSION >= (1, 7) else
                            {'nullable': True, 'outer_if_first': True})
                alias = self.query.join((self.query.get_initial_alias(), self.model._meta.db_table,
                                        ((masteratt, masteratt),)),
                                        join_field=BetterTranslationsField(languages, master=masteratt),
                                        **nullable)

            add_alias_constraints(self, (self.model, alias), id__isnull=True)
            self.query.add_filter(('%s__isnull' % masteratt, False))
            if not self._skip_master_select and getattr(self, '_fields', None) is None:
                self.query.add_select_related(('master',))

        else:
            language_code = self._language_code or get_language()
            for _, field_name in where_node_children(self.query.where):
                if field_name == 'language_code':
                    # remove in 1.4
                    raise RuntimeError(
                        'Overriding language_code in get() or filter() is no longer supported. '
                        'Please set the language in Model.objects.language() instead, '
                        'or use language("all") to do manual filtering on languages.')
            else:
                self.query.add_filter(('language_code', language_code))
            self._add_select_related(language_code)

        # if queryset is about to use the model's default ordering, we
        # override that now with a translated version of the model's ordering
        if self.query.default_ordering and not self.query.order_by:
            ordering = self.shared_model._meta.ordering
            self.query.order_by = self._translate_fieldnames(ordering or [])

        return self

    def _use_related_translations(self, obj, relations_dict, depth=0):
        """
        Ensure that we use cached translations brought in via select_related if
        available. Necessary since the database select_related query caches the
        related translation models in a different place than hvad expects it.
        """

        # First, set translation for current object,
        accessor = getattr(obj._meta, 'translations_accessor', None)
        if accessor is not None:
            if django.VERSION >= (1, 9):
                cache = getattr(obj.__class__, accessor).rel.get_cache_name()
            else:
                cache = getattr(obj.__class__, accessor).related.get_cache_name()
            try:
                translation = getattr(obj, cache)
            except AttributeError:
                pass
            else:
                delattr(obj, cache)
                setattr(obj, obj._meta.translations_cache, translation)

        # Then recurse in the relation dict
        for field, sub_dict in relations_dict.items():
            target = translation if field == accessor else getattr(obj, field)
            if target is not None:
                self._use_related_translations(target, sub_dict, depth+1)


    #===========================================================================
    # Queryset/Manager API
    #===========================================================================

    def language(self, language_code=None):
        self._language_code = language_code
        return self

    def fallbacks(self, *fallbacks):
        if not fallbacks:
            self._language_fallbacks = FALLBACK_LANGUAGES
        elif fallbacks == (None,):
            self._language_fallbacks = None
        else:
            self._language_fallbacks = fallbacks
        return self

    #===========================================================================
    # Queryset/Manager API that do database queries
    #===========================================================================

    def iterator(self):
        """
        If this queryset is not filtered by a language code yet, it should be
        filtered first by calling self.language.

        If someone doesn't want a queryset filtered by language, they should use
        Model.objects.untranslated()
        """
        qs = self._clone()._add_language_filter()
        qs._known_related_objects = {}  # super's iterator will attempt to set them

        if django.VERSION >= (1, 9) and qs._iterable_class in iterable_overrides:
            qs._iterable_class = iterable_overrides[qs._iterable_class]
            for obj in super(TranslationQueryset, qs).iterator():
                yield obj
            return

        if qs._forced_unique_fields:
            # HACK: In order for select_related to properly load data from
            # translated models, we have to force django to treat
            # certain fields as one-to-one relations
            # before this queryset calls get_cached_row()
            # We change it back so that things get reset to normal
            # before execution returns to user code.
            # It would be more direct and robust if we could wrap
            # django.db.models.query.get_cached_row() instead, but that's not a class
            # method, sadly, so we cannot override it just for this query

            with ForcedUniqueFields(qs._forced_unique_fields):
                # Pre-fetch all objects:
                objects = list(super(TranslationQueryset, qs).iterator())

            if type(qs.query.select_related) == dict:
                for obj in objects:
                    qs._use_related_translations(obj, qs.query.select_related)
        else:
            objects = super(TranslationQueryset, qs).iterator()

        for obj in objects:
            for name in self._hvad_switch_fields:
                try:
                    setattr(obj.master, name, getattr(obj, name))
                except AttributeError: # pragma: no cover
                    pass
                else:
                    delattr(obj, name)
            obj = combine(obj, qs.shared_model)
            # use known objects from self, not qs as we cleared it earlier
            for field, rel_objs in self._known_related_objects.items():
                if hasattr(obj, field.get_cache_name()):
                    # should not happen, but we conform to Django behavior
                    continue #pragma: no cover
                pk = getattr(obj, field.get_attname())
                try:
                    rel_obj = rel_objs[pk]
                except KeyError: #pragma: no cover
                    pass
                else:
                    setattr(obj, field.name, rel_obj)
            yield obj

    def create(self, **kwargs):
        if 'language_code' in kwargs:
            if self._language_code is not None:
                # remove in 1.4
                raise RuntimeError('Overriding language_code in create() is no longer allowed. '
                                   'Please set the language with language() instead.')
        else:
            kwargs['language_code'] = self._language_code or get_language()
        if kwargs['language_code'] == 'all':
            raise ValueError('Cannot create an object with language \'all\'')
        obj = self.shared_model(**kwargs)
        self._for_write = True
        obj.save(force_insert=True, using=self.db)
        return obj

    def count(self):
        if self._result_cache is None:
            qs = self._clone()._add_language_filter()
            return super(TranslationQueryset, qs).count()
        else:
            return len(self._result_cache)

    def exists(self):
        if self._result_cache is None:
            qs = self._clone()._add_language_filter()
            return super(TranslationQueryset, qs).exists()
        else:
            return bool(self._result_cache)

    def get_or_create(self, **kwargs):
        """
        Looks up an object with the given kwargs, creating one if necessary.
        Returns a tuple of (object, created), where created is a boolean
        specifying whether an object was created.
        """
        assert kwargs, \
                'get_or_create() must be passed at least one keyword argument'
        defaults = kwargs.pop('defaults', {})
        lookup = kwargs.copy()
        for f in self.model._meta.fields:
            if f.attname in lookup:
                lookup[f.name] = lookup.pop(f.attname)
        try:
            self._for_write = True
            return self.get(**lookup), False
        except self.model.DoesNotExist:
            pass

        params = dict([(k, v) for k, v in kwargs.items() if '__' not in k])
        params.update(defaults)

        if 'language_code' in params:
            if self._language_code is not None:
                # remove in 1.4
                raise RuntimeError('Overriding language_code in get_or_create() is no longer allowed. '
                                   'Please set the language in Model.objects.language() instead.')
        else:
            params['language_code'] = self._language_code or get_language()
        if params['language_code'] == 'all':
            raise ValueError('Cannot create an object with language \'all\'')

        obj = self.shared_model(**params)
        try:
            with transaction.atomic(using=self.db):
                obj.save(force_insert=True, using=self.db)
            return obj, True
        except IntegrityError:
            exc_info = sys.exc_info()
            try:
                return self.get(**lookup), False
            except self.model.DoesNotExist:
                raise exc_info[1]

    @minimumDjangoVersion(1, 7)
    def update_or_create(self, defaults=None, **kwargs):
        raise NotImplementedError()

    def bulk_create(self, objs, batch_size=None):
        raise NotImplementedError()

    def aggregate(self, *args, **kwargs):
        """
        Loops over all the passed aggregates and translates the fieldnames
        """
        qs = self._clone()._add_language_filter()
        newargs = tuple(qs._translate_expression(item) for item in args)
        newkwargs = dict(
            (k, qs._translate_expression(v)) for k, v in kwargs.items()
        )
        response = super(TranslationQueryset, qs).aggregate(*newargs, **newkwargs)
        return qs._reverse_translate_fieldnames_dict(response)

    def latest(self, field_name=None):
        field_name = self.field_translator(field_name or self.shared_model._meta.get_latest_by)
        return super(TranslationQueryset, self).latest(field_name)

    def earliest(self, field_name=None):
        field_name = self.field_translator(field_name or self.shared_model._meta.get_latest_by)
        return super(TranslationQueryset, self).earliest(field_name)

    def in_bulk(self, id_list):
        if not id_list:
            return {}
        if self._language_code == 'all':
            raise ValueError('Cannot use in_bulk along with language(\'all\').')
        qs = self.filter(pk__in=id_list)
        qs.query.clear_ordering(force_empty=True)
        return dict((obj._get_pk_val(), obj) for obj in qs.iterator())

    def delete(self):
        qs = self._get_shared_queryset()
        qs.delete()
    delete.alters_data = True
    delete.queryset_only = True

    def delete_translations(self):
        if connections[self._db].features.update_can_self_select:
            qs = self._clone()._add_language_filter()
            super(TranslationQueryset, qs).delete()
        else:
            with transaction.atomic(using=self._db, savepoint=False):
                pks = list(super(TranslationQueryset, self).values_list('pk', flat=True))
                self.model._base_manager.filter(pk__in=pks).delete()
    delete_translations.alters_data = True

    def update(self, **kwargs):
        qs = self._clone()._add_language_filter()
        shared, translated = qs._split_kwargs(**kwargs)
        count = 0
        if translated:
            count += super(TranslationQueryset, qs).update(**translated)
        if shared:
            shared_qs = qs._get_shared_queryset()
            count += shared_qs.update(**shared)
        return count
    update.alters_data = True

    #===========================================================================
    # Queryset/Manager API that return another queryset
    #===========================================================================

    def filter(self, *args, **kwargs):
        if 'language_code' in kwargs and kwargs['language_code'] == 'all':
            raise ValueError('Value "all" is invalid for language_code')
        newargs, newkwargs = self._translate_args_kwargs(*args, **kwargs)
        return super(TranslationQueryset, self).filter(*newargs, **newkwargs)

    def exclude(self, *args, **kwargs):
        if 'language_code' in kwargs and kwargs['language_code'] == 'all':
            raise ValueError('Value "all" is invalid for language_code')
        newargs, newkwargs = self._translate_args_kwargs(*args, **kwargs)
        return super(TranslationQueryset, self).exclude(*newargs, **newkwargs)

    def extra(self, select=None, where=None, params=None, tables=None,
              order_by=None, select_params=None):
        qs = super(TranslationQueryset, self).extra(select, where, params, tables,
                                                    order_by, select_params)
        if select:
            switch_fields = set(self._hvad_switch_fields)
            switch_fields.update(select.keys())
            qs._hvad_switch_fields = tuple(switch_fields)
        return qs

    def values(self, *fields):
        fields = self._translate_fieldnames(fields)
        return super(TranslationQueryset, self).values(*fields)

    def values_list(self, *fields, **kwargs):
        fields = self._translate_fieldnames(fields)
        return super(TranslationQueryset, self).values_list(*fields, **kwargs)

    def dates(self, field_name, kind=None, order='ASC'):
        if django.VERSION < (1, 8):
            field_name = self.field_translator(field_name)
        return super(TranslationQueryset, self).dates(field_name, kind=kind, order=order)

    def datetimes(self, field, *args, **kwargs):
        if django.VERSION < (1, 8):
            field = self.field_translator(field)
        return super(TranslationQueryset, self).datetimes(field, *args, **kwargs)

    def select_related(self, *fields):
        if not fields:
            raise NotImplementedError('To use select_related on a translated model, '
                                      'you must provide a list of fields.')
        if fields == (None,):
            self._raw_select_related = []
        elif django.VERSION >= (1, 7):  # in newer versions, calls are cumulative
            self._raw_select_related.extend(fields)
        else:  #pragma: no cover        # in older versions, they overwrite each other
            self._raw_select_related = list(fields)
        return self

    def complex_filter(self, filter_obj):
        # Don't know how to handle Q object yet, but it is probably doable...
        # An unknown type object that supports 'add_to_query' is a different story :)
        if isinstance(filter_obj, models.Q) or hasattr(filter_obj, 'add_to_query'):
            raise NotImplementedError()

        newargs, newkwargs = self._translate_args_kwargs(**filter_obj)
        return super(TranslationQueryset, self)._filter_or_exclude(None, *newargs, **newkwargs)

    def annotate(self, *args, **kwargs):
        newkwargs = dict((k, self._translate_expression(v)) for k, v in kwargs.items())
        for arg in args:
            arg = self._translate_expression(arg)
            try:
                alias = arg.default_alias
            except (AttributeError, TypeError):
                raise TypeError("Complex annotations require an alias")
            if alias.startswith('master__'):
               alias = alias[8:]
            if alias in kwargs:
                raise ValueError("The named annotation '%s' conflicts with the "
                                 "default name for another annotation." % alias)
            newkwargs[alias] = arg

        qs = super(TranslationQueryset, self).annotate(**newkwargs)

        switch_fields = set(qs._hvad_switch_fields)
        switch_fields.update(newkwargs)
        qs._hvad_switch_fields = tuple(switch_fields)
        return qs

    def order_by(self, *field_names):
        """
        Returns a new QuerySet instance with the ordering changed.
        """
        fieldnames = self._translate_fieldnames(field_names)
        return super(TranslationQueryset, self).order_by(*fieldnames)

    def reverse(self):
        return super(TranslationQueryset, self).reverse()

    def defer(self, *fields):
        raise NotImplementedError()

    def only(self, *fields):
        raise NotImplementedError()

#===============================================================================
# Fallbacks
#===============================================================================

class _SharedFallbackQueryset(QuerySet):
    translation_fallbacks = None

    def use_fallbacks(self, *fallbacks):
        if django.VERSION >= (1, 9):
            raise AssertionError(
                'TranslationManager.untranslated().use_fallbacks() is not supported on Django 1.9, '
                'and is deprecated on previous versions as well. Please use the fallbacks() method '
                'on queryset instead, eg: MyModel.objects.language().fallbacks()')
        warnings.warn('TranslationManager.untranslated().use_fallbacks() is deprecated. '
                      'Please use the fallbacks() method on queryset instead, eg: '
                      'MyModel.objects.language().fallbacks()', DeprecationWarning, stacklevel=2)
        self.translation_fallbacks = fallbacks or (None,)+FALLBACK_LANGUAGES
        return self

    def _clone(self, klass=None, setup=False, **kwargs):
        kwargs.update({
            'translation_fallbacks': self.translation_fallbacks,
        })
        if django.VERSION < (1, 9):
            kwargs.update({'klass': klass, 'setup': setup})
        return super(_SharedFallbackQueryset, self)._clone(**kwargs)

    def aggregate(self, *args, **kwargs):
        raise NotImplementedError()

    def annotate(self, *args, **kwargs):
        raise NotImplementedError()

    def defer(self, *args, **kwargs):
        raise NotImplementedError()

    def only(self, *args, **kwargs):
        raise NotImplementedError()


class LegacyFallbackQueryset(_SharedFallbackQueryset): #pragma: no cover
    '''
    Queryset that tries to load a translated version using fallbacks on a per
    instance basis.
    BEWARE: creates a lot of queries!
    '''
    def _get_real_instances(self, base_results):
        """
        The logic for this method was taken from django-polymorphic by Bert
        Constantin (https://github.com/bconstantin/django_polymorphic) and was
        slightly altered to fit the needs of django-hvad.
        """
        # get the primary keys of the shared model results
        base_ids = [obj.pk for obj in base_results]
        fallbacks = [get_language() if lang is None else lang
                     for lang in self.translation_fallbacks]
        # get all translations for the fallbacks chosen for those shared models,
        # note that this query is *BIG* and might return a lot of data, but it's
        # arguably faster than running one query for each result or even worse
        # one query per result per language until we find something
        translations_manager = self.model._meta.translations_model.objects
        baseqs = translations_manager.select_related('master')
        translations = baseqs.filter(language_code__in=fallbacks,
                                     master__pk__in=base_ids)
        fallback_objects = defaultdict(dict)
        # turn the results into a dict of dicts with shared model primary key as
        # keys for the first dict and language codes for the second dict
        for obj in translations:
            fallback_objects[obj.master.pk][obj.language_code] = obj
        # iterate over the share dmodel results
        for instance in base_results:
            translation = None
            # find the translation
            for fallback in fallbacks:
                translation = fallback_objects[instance.pk].get(fallback, None)
                if translation is not None:
                    break
            # if we found a translation, yield the combined result
            if translation:
                yield combine(translation, self.model)
            else:
                # otherwise yield the shared instance only
                _logger.error("no translation for %s.%s (pk=%s)" %
                              (instance._meta.app_label,
                               instance.__class__.__name__,
                               str(instance.pk)))
                yield instance

    def iterator(self):
        """
        The logic for this method was taken from django-polymorphic by Bert
        Constantin (https://github.com/bconstantin/django_polymorphic) and was
        slightly altered to fit the needs of django-hvad.
        """
        base_iter = super(LegacyFallbackQueryset, self).iterator()

        # only do special stuff when we actually want fallbacks
        if self.translation_fallbacks:
            while True:
                base_result_objects = []
                reached_end = False

                # get the next "chunk" of results
                for i in range(CHUNK_SIZE):
                    try:
                        instance = next(base_iter)
                        base_result_objects.append(instance)
                    except StopIteration:
                        reached_end = True
                        break

                # "combine" the results with their fallbacks
                real_results = self._get_real_instances(base_result_objects)

                # yield em!
                for instance in real_results:
                    yield instance

                # get out of the while loop if we're at the end, since this is
                # an iterator, we need to raise StopIteration, not "return".
                if reached_end:
                    raise StopIteration
        else:
            # just iterate over it
            for instance in base_iter:
                yield instance

class SelfJoinFallbackQueryset(_SharedFallbackQueryset):
    def iterator(self):
        # only do special stuff when we actually want fallbacks
        if self.translation_fallbacks:
            fallbacks = [get_language() if lang is None else lang
                         for lang in self.translation_fallbacks]
            tmodel = self.model._meta.translations_model
            taccessor = self.model._meta.translations_accessor
            taccessorcache = (
                getattr(self.model, taccessor).rel.get_cache_name() if django.VERSION >= (1, 9) else
                getattr(self.model, taccessor).related.get_cache_name()
            )
            tcache = self.model._meta.translations_cache
            masteratt = tmodel._meta.get_field('master').attname
            field = BetterTranslationsField(fallbacks, master=masteratt)

            qs = self._clone()

            qs.query.add_select_related((taccessor,))
            # This join will be reused by the select_related. We must provide it
            # anyway because the order matters and add_select_related does not
            # populate joins right away.
            if django.VERSION >= (1, 8):
                alias1 = qs.query.join(Join(
                    tmodel._meta.db_table,
                    qs.query.get_initial_alias(),
                    None,
                    LOUTER,
                    getattr(qs.model, taccessor).field.rel if django.VERSION >= (1, 9) else
                    getattr(qs.model, taccessor).related.field.rel,
                    True
                ))
                alias2 = qs.query.join(Join(
                    tmodel._meta.db_table,
                    alias1,
                    None,
                    LOUTER,
                    field,
                    True
                ))
            else:
                nullable = ({'nullable': True} if django.VERSION >= (1, 7) else
                            {'nullable': True, 'outer_if_first': True})
                alias1 = qs.query.join((qs.query.get_initial_alias(), tmodel._meta.db_table,
                                        ((qs.model._meta.pk.attname, masteratt),)),
                                    join_field=getattr(qs.model, taccessor).related.field.rel,
                                    **nullable)
                alias2 = qs.query.join((tmodel._meta.db_table, tmodel._meta.db_table,
                                        ((masteratt, masteratt),)),
                                    join_field=field, **nullable)

            add_alias_constraints(qs, (tmodel, alias2), id__isnull=True)

            # We must force the _unique field so get_cached_row populates the cache
            # Unfortunately, this means we must load everything in one go
            related_field = (getattr(qs.model, taccessor).field if django.VERSION >= (1, 9) else
                             getattr(qs.model, taccessor).related.field)
            with ForcedUniqueFields([related_field]):
                objects = []
                for instance in super(SelfJoinFallbackQueryset, qs).iterator():
                    try:
                        translation = getattr(instance, taccessorcache)
                    except AttributeError: #pragma: no cover
                        _logger.error("no translation for %s.%s (pk=%s)",
                                    instance._meta.app_label,
                                    instance.__class__.__name__,
                                    str(instance.pk))
                    else:
                        setattr(instance, tcache, translation)
                        delattr(instance, taccessorcache)
                    objects.append(instance)
            return iter(objects)
        else:
            return super(SelfJoinFallbackQueryset, self).iterator()


FallbackQueryset = LegacyFallbackQueryset if LEGACY_FALLBACKS else SelfJoinFallbackQueryset


#===============================================================================
# TranslationManager
#===============================================================================


class TranslationManager(models.Manager):
    """
    Manager class for models with translated fields
    """
    #===========================================================================
    # API
    #===========================================================================
    use_for_related_fields = True
    silence_use_for_related_fields_deprecation = True   # Django 1.10

    queryset_class = TranslationQueryset
    fallback_class = FallbackQueryset
    default_class = QuerySet

    def __init__(self, *args, **kwargs):
        self.queryset_class = kwargs.pop('queryset_class', self.queryset_class)
        self.fallback_class = kwargs.pop('fallback_class', self.fallback_class)
        self.default_class = kwargs.pop('default_class', self.default_class)
        super(TranslationManager, self).__init__(*args, **kwargs)

    def _make_queryset(self, klass, core_filters):
        ''' Builds a queryset of given class.
            core_filters tells whether the queryset will bypass RelatedManager
            mechanics and therefore needs to reapply the filters on its own.
        '''
        if django.VERSION >= (1, 7):
            qs = klass(self.model, using=self.db, hints=self._hints)
        else:
            qs = klass(self.model, using=self.db) #pragma: no cover
        core_filters = getattr(self, 'core_filters', None) if core_filters else None
        if core_filters:
            qs = qs._next_is_sticky().filter(**core_filters)
        return qs

    def language(self, language_code=None):
        return self._make_queryset(self.queryset_class, True).language(language_code)

    def untranslated(self):
        return self._make_queryset(self.fallback_class, True)

    def get_queryset(self):
        return self._make_queryset(self.default_class, False)

    #===========================================================================
    # Internals
    #===========================================================================

    @property
    def translations_model(self):
        return self.model._meta.translations_model


#===============================================================================
# TranslationAware
#===============================================================================


class TranslationAwareQueryset(QuerySet):
    def __init__(self, *args, **kwargs):
        super(TranslationAwareQueryset, self).__init__(*args, **kwargs)
        self._language_code = None

    def _translate(self, key, model, language_joins):
        ''' Translate a key on a model.
            language_joins must be a set that will be updated with required
            language joins for given key
        '''
        newkey = []
        for term in query_terms(model, key):
            if term.translated:
                newkey.append('%s__%s' % (term.model._meta.translations_accessor, term.term))
            else:
                newkey.append(term.term)
            if term.target is not None:
                taccessor = getattr(term.target._meta, 'translations_accessor', None)
                if taccessor is not None:
                    language_joins.add('__'.join(newkey + [taccessor, 'language_code']))
        return '__'.join(newkey)

    def _translate_args_kwargs(self, *args, **kwargs):
        self.language(self._language_code)
        language_joins = set()
        extra_filters = Q()
        newkwargs = dict(
            (self._translate(key, self.model, language_joins), value)
            for key, value in kwargs.items()
        )
        newargs = deepcopy(args)
        for q in newargs:
            for child, children, index in q_children(q):
                children[index] = (self._translate(child[0], self.model, language_joins),
                                   child[1])
        for langjoin in language_joins:
            extra_filters &= Q(**{langjoin: self._language_code})
        return newargs, newkwargs, extra_filters

    def _translate_fieldnames(self, fields):
        self.language(self._language_code)
        extra_filters = Q()
        language_joins = set()
        newfields = tuple(
            self._translate(field, self.model, language_joins)
            for field in fields
        )

        for langjoin in language_joins:
            extra_filters &= Q(**{langjoin: self._language_code})
        return newfields, extra_filters

    #===========================================================================
    # Queryset/Manager API
    #===========================================================================

    def language(self, language_code=None):
        if not language_code:
            language_code = get_language()
        self._language_code = language_code
        return self

    def get(self, *args, **kwargs):
        newargs, newkwargs, extra_filters = self._translate_args_kwargs(*args, **kwargs)
        return self._filter_extra(extra_filters).get(*newargs, **newkwargs)

    def filter(self, *args, **kwargs):
        newargs, newkwargs, extra_filters = self._translate_args_kwargs(*args, **kwargs)
        return self._filter_extra(extra_filters).filter(*newargs, **newkwargs)

    def aggregate(self, *args, **kwargs):
        raise NotImplementedError()

    def latest(self, field_name=None):
        extra = Q()
        if field_name:
            language_joins, lang = set(), self._language_code or get_language()
            field_name = self._translate(field_name, self.model, language_joins)
            for join in language_joins:
                extra &= Q(**{join: lang})
        return self._filter_extra(extra).latest(field_name)

    def earliest(self, field_name=None):
        extra = Q()
        if field_name:
            language_joins, lang = set(), self._language_code or get_language()
            field_name = self._translate(field_name, self.model, language_joins)
            for join in language_joins:
                extra &= Q(**{join: lang})
        return self._filter_extra(extra).earliest(field_name)

    def in_bulk(self, id_list):
        if not id_list:
            return {}
        qs = self.filter(pk__in=id_list)
        qs.query.clear_ordering(force_empty=True)
        return dict((obj._get_pk_val(), obj) for obj in qs.iterator())

    def values(self, *fields):
        fields, extra_filters = self._translate_fieldnames(fields)
        return self._filter_extra(extra_filters).values(*fields)

    def values_list(self, *fields, **kwargs):
        fields, extra_filters = self._translate_fieldnames(fields)
        return self._filter_extra(extra_filters).values_list(*fields, **kwargs)

    def dates(self, field_name, kind, order='ASC'):
        raise NotImplementedError()

    def datetimes(self, field, *args, **kwargs):
        raise NotImplementedError()

    def exclude(self, *args, **kwargs):
        newargs, newkwargs, extra_filters = self._translate_args_kwargs(*args, **kwargs)
        if extra_filters.children:
            combined = extra_filters & ~Q(*args, **kwargs)
            return super(TranslationAwareQueryset, self).filter(combined)
        else:
            return super(TranslationAwareQueryset, self).exclude(*newargs, **newkwargs)

    def complex_filter(self, filter_obj):
        # admin calls this with an empy filter_obj sometimes
        if filter_obj == {}:
            return self
        raise NotImplementedError()

    def annotate(self, *args, **kwargs):
        raise NotImplementedError()

    def order_by(self, *field_names):
        """
        Returns a new QuerySet instance with the ordering changed.
        """
        fieldnames, extra_filters = self._translate_fieldnames(field_names)
        return self._filter_extra(extra_filters).order_by(*fieldnames)

    def reverse(self):
        raise NotImplementedError()

    def defer(self, *fields):
        raise NotImplementedError()

    def only(self, *fields):
        raise NotImplementedError()

    def _clone(self, klass=None, setup=False, **kwargs):
        kwargs.update({
            '_language_code': self._language_code,
        })
        if django.VERSION < (1, 9):
            kwargs.update({'klass': klass, 'setup': setup})
        return super(TranslationAwareQueryset, self)._clone(**kwargs)

    def _filter_extra(self, extra_filters):
        if extra_filters.children:
            qs = self._next_is_sticky()
            qs = super(TranslationAwareQueryset, qs).filter(extra_filters)
        else:
            qs = self
        return super(TranslationAwareQueryset, qs)


class TranslationAwareManager(models.Manager):
    def language(self, language_code=None):
        return self.get_queryset().language(language_code)

    def get_queryset(self):
        qs = TranslationAwareQueryset(self.model, using=self.db)
        return qs

#===============================================================================
# Translations Model Manager
#===============================================================================


class TranslationsModelManager(models.Manager):
    def get_language(self, language):
        qs = self.all()
        if qs._result_cache is None:
            return self.get(language_code=language)
        else: # take advantage of cached translations
            for obj in qs:
                if obj.language_code == language:
                    return obj
        raise self.model.DoesNotExist



import sys
PYTHON_MAJOR, PYTHON_MINOR = sys.version_info[0:2]

__all__ = ('with_metaclass', 'MethodType', 'StringIO', 'string_types',
           'urlencode', 'urlparse', 'unquote')

#=============================================================================

def with_metaclass(meta, *bases):
    ''' Python 2/3 cross-compatible way to use a metclass
        Remove when support for Python 2 is dropped.
    '''
    class metaclass(meta):
        def __new__(cls, name, this_bases, d):
            return meta(name, bases, d)
    return type.__new__(metaclass, 'temporary_class', (), {})

#=============================================================================

if PYTHON_MAJOR == 2:
    from types import MethodType as OriginalMethodType
    def MethodType(function, instance):
        return OriginalMethodType(function, instance, instance.__class__)

    string_types = (str, unicode)
    from urllib import urlencode
    from urlparse import urlparse
    from urllib import unquote

    if PYTHON_MINOR >= 6:
        from io import StringIO
    else: #pragma: no cover
        from StringIO import StringIO

else:
    from types import MethodType
    string_types = (str,)
    from urllib.parse import urlencode, urlparse, unquote
    from io import StringIO




import django
from django.conf import settings
from django.core.exceptions import FieldError, ValidationError
from django.forms.fields import CharField
from django.forms.formsets import formset_factory
from django.forms.models import (ModelForm, BaseModelForm, ModelFormMetaclass,
    fields_for_model, model_to_dict, construct_instance, BaseInlineFormSet, BaseModelFormSet,
    modelform_factory, inlineformset_factory)
if django.VERSION >= (1, 7):
    from django.forms.utils import ErrorList
else: #pragma: no cover
    from django.forms.util import ErrorList
from django.forms.widgets import Select
from django.utils.translation import get_language, ugettext as _
from hvad.compat import with_metaclass
from hvad.models import TranslatableModel, BaseTranslationModel
from hvad.utils import (set_cached_translation, get_cached_translation, load_translation)
try:
    from collections import OrderedDict
except ImportError: #pragma: no cover (python < 2.7)
    from django.utils.datastructures import SortedDict as OrderedDict
import warnings

veto_fields = ('id', 'master', 'master_id', 'language_code')

#=============================================================================

class TranslatableModelFormMetaclass(ModelFormMetaclass):
    ''' Metaclass used for forms with translatable fields.
        It wraps the regular ModelFormMetaclass to intercept translatable fields,
        otherwise it would choke. Translatable fields are then inserted back
        into the created class.
    '''
    def __new__(cls, name, bases, attrs):
        # Force presence of meta class, we need it
        meta = attrs.get('Meta')
        if meta is None:
            # if a base class has a Meta, inherit it
            base_meta = next(((base.Meta,) for base in bases if hasattr(base, 'Meta')), ())
            meta = attrs['Meta'] = type('Meta', base_meta + (object,), {})

        model = getattr(meta, 'model', None)
        fields = getattr(meta, 'fields', None)

        # Force exclusion of language_code as we use cleaned_data['language_code']
        exclude = meta.exclude = list(getattr(meta, 'exclude', ()))
        if fields is not None and 'language_code' in fields:
            raise FieldError('Field \'language_code\' is invalid.')

        # If a model is provided, handle translatable fields
        if model:
            if not issubclass(model, TranslatableModel):
                raise TypeError('TranslatableModelForm only works with TranslatableModel'
                                ' subclasses, which %s is not.' % model.__name__)

            # Additional exclusions
            exclude.append(model._meta.translations_accessor)
            if fields is not None and model._meta.translations_accessor in fields:
                raise FieldError('Field \'%s\' is invalid', model._meta.translations_accessor)

            # Get translatable fields
            tfields = fields_for_model(
                model._meta.translations_model,
                fields=fields,
                exclude=exclude + list(veto_fields),
                widgets=getattr(meta, 'widgets', None),
                formfield_callback=attrs.get('formfield_callback')
            )

            # Drop translatable fields from Meta.fields
            if fields is not None:
                meta.fields = [field for field in fields if tfields.get(field) is None]

        # Create the form class
        new_class = super(TranslatableModelFormMetaclass, cls).__new__(cls, name, bases, attrs)

        # Add translated fields into the form's base fields
        if model:
            if fields is None:
                # loop, as Django's variant of OrderedDict cannot consume generators
                for name, field in tfields.items():
                    if field is not None:
                        new_class.base_fields[name] = field
            else:
                # rebuild the fields to respect Meta.fields ordering
                new_class.base_fields = OrderedDict(
                    item for item in (
                        (name, new_class.base_fields.get(name, tfields.get(name)))
                        for name in fields
                    )
                    if item[1] is not None
                )
                # restore hijacked Meta.fields
                new_class._meta.fields = meta.fields = fields
        return new_class


#=============================================================================

class BaseTranslatableModelForm(BaseModelForm):
    ''' Base class for forms dealing with TranslatableModel
        It has two main responsibilities: loading translated fields into the form
        when __init__ialized and ensuring the right translation gets saved.
    '''
    def __init__(self, data=None, files=None, auto_id='id_%s', prefix=None,
                 initial=None, error_class=ErrorList, label_suffix=':',
                 empty_permitted=False, instance=None, **kwargs):

        # Insert values of instance's translated fields into 'initial' dict
        object_data = {}
        enforce = hasattr(self, 'language')
        language = getattr(self, 'language', None) or get_language()

        if instance is not None:
            translation = load_translation(instance, language, enforce)
            if translation.pk:
                exclude = (tuple(self._meta.exclude or ()) + veto_fields)
                object_data.update(
                    model_to_dict(translation, self._meta.fields, exclude)
                )
        if initial is not None:
            object_data.update(initial)

        super(BaseTranslatableModelForm, self).__init__(
            data, files, auto_id, prefix, object_data,
            error_class, label_suffix, empty_permitted, instance, **kwargs
        )

    def clean(self):
        ''' If a language is set on the form, enforce it by overwriting it
            in the cleaned_data.
        '''
        data = super(BaseTranslatableModelForm, self).clean()
        if hasattr(self, 'language'):
            data['language_code'] = self.language
        return data

    def _post_clean(self):
        ''' Switch the translation on self.instance
            This cannot (and should not) be done in clean() because it could be
            overriden to change the language. Yet it should be done before save()
            to allow an overriden save to set some translated field values before
            invoking super().
        '''
        enforce = 'language_code' in self.cleaned_data
        language = self.cleaned_data.get('language_code') or get_language()
        translation = load_translation(self.instance, language, enforce)

        exclude = self._get_validation_exclusions()
        translation = construct_instance(self, translation, self._meta.fields, exclude)
        set_cached_translation(self.instance, translation)
        result = super(BaseTranslatableModelForm, self)._post_clean()
        return result

    def _get_validation_exclusions(self):
        exclude = super(BaseTranslatableModelForm, self)._get_validation_exclusions()
        for f in self.instance._meta.translations_model._meta.fields:
            if f.name in veto_fields:
                pass
            elif ((f.name not in self.fields) or
                (self._meta.fields and f.name not in self._meta.fields) or
                (self._meta.exclude and f.name in self._meta.exclude) or
                (f.name in self._errors)):
                exclude.append(f.name)
            else:
                form_field = self.fields[f.name]
                field_value = self.cleaned_data.get(f.name, None)
                if not f.blank and not form_field.required and field_value in form_field.empty_values:
                    exclude.append(f.name)
        return exclude

    def save(self, commit=True):
        ''' Saves the model
            If will always use the language specified in self.cleaned_data, with
            the usual None meaning 'call get_language()'. If instance has
            another language loaded, it gets reloaded with the new language.

            If no language is specified in self.cleaned_data, assume the instance
            is preloaded with correct language.
        '''
        assert self.is_valid(), ('Method save() must not be called on an invalid '
                                 'form. Check the result of .is_valid() before '
                                 'calling save().')

        # Get the right translation for object and language
        # It should have been done in _post_clean, but instance may have been
        # changed since.
        enforce = 'language_code' in self.cleaned_data
        language = self.cleaned_data.get('language_code') or get_language()
        translation = load_translation(self.instance, language, enforce)

        # Fill the translated fields with values from the form
        excludes = list(self._meta.exclude) + ['master', 'language_code']
        translation = construct_instance(self, translation,
                                         self._meta.fields, excludes)
        set_cached_translation(self.instance, translation)

        # Delegate shared fields to super()
        return super(BaseTranslatableModelForm, self).save(commit=commit)


if django.VERSION >= (1, 7):
    class TranslatableModelForm(with_metaclass(TranslatableModelFormMetaclass,
                                               BaseTranslatableModelForm)):
        pass
else: #pragma: no cover
    # Older django version have buggy metaclass
    class TranslatableModelForm(with_metaclass(TranslatableModelFormMetaclass,
                                               BaseTranslatableModelForm, ModelForm)):
        __metaclass__ = TranslatableModelFormMetaclass # Django 1.4 compatibility


#=============================================================================

def translatable_modelform_factory(language, model, form=TranslatableModelForm, *args, **kwargs):
    if not issubclass(form, TranslatableModelForm):
        raise TypeError('The form class given to translatable_modelform_factory '
                        'must be a subclass of hvad.forms.TranslatableModelForm')
    klass = modelform_factory(model, form, *args, **kwargs)
    klass.language = language
    return klass


def translatable_modelformset_factory(language, model, form=TranslatableModelForm,
                                      formfield_callback=None, formset=BaseModelFormSet,
                                      extra=1, can_delete=False, can_order=False,
                                      max_num=None, fields=None, exclude=None, **kwargs):

    # This Django API changes often, handle args we know and raise for others
    form_kwargs, formset_kwargs = {}, {}
    for key in ('widgets', 'localized_fields', 'labels', 'help_texts', 'error_messages'):
        if key in kwargs:
            form_kwargs[key] = kwargs.pop(key)
    for key in ('validate_max',):
        if key in kwargs:
            formset_kwargs[key] = kwargs.pop(key)
    if kwargs:
        raise TypeError('Unknown arguments %r for translatable_modelformset_factory. '
                        'If it is legit, it is probably new in Django. Please open '
                        'a ticket so we can add it.' % tuple(kwargs.keys()))

    form = translatable_modelform_factory(
        language, model, form=form, fields=fields, exclude=exclude,
        formfield_callback=formfield_callback, **form_kwargs
    )
    FormSet = formset_factory(form, formset, extra=extra, max_num=max_num,
                              can_order=can_order, can_delete=can_delete, **formset_kwargs)
    FormSet.model = model
    return FormSet


def translatable_inlineformset_factory(language, parent_model, model, form=TranslatableModelForm,
                                       formset=BaseInlineFormSet, fk_name=None,
                                       fields=None, exclude=None, extra=3,
                                       can_order=False, can_delete=True,
                                       max_num=None, formfield_callback=None, **kwargs):
    from django.forms.models import _get_foreign_key
    fk = _get_foreign_key(parent_model, model, fk_name=fk_name)
    if fk.unique:  #pragma: no cover (internal Django behavior)
        max_num = 1

    FormSet = translatable_modelformset_factory(language, model,
         form=form, formfield_callback=formfield_callback, formset=formset,
         extra=extra, can_delete=can_delete, can_order=can_order,
         fields=fields, exclude=exclude, max_num=max_num, **kwargs)
    FormSet.fk = fk
    return FormSet


#=============================================================================

class BaseTranslationFormSet(BaseInlineFormSet):
    """A kind of inline formset for working with an instance's translations.
    It keeps track of the real object and combine()s it to the translations
    for validation and saving purposes.
    It can delete translations, but will refuse to delete the last one.
    """
    def __init__(self, *args, **kwargs):
        super(BaseTranslationFormSet, self).__init__(*args, **kwargs)
        self.queryset = self.order_translations(self.queryset)

    def order_translations(self, qs):
        return qs.order_by('language_code')

    def clean(self):
        super(BaseTranslationFormSet, self).clean()

        # Trigger combined instance validation
        master = self.instance
        stashed = get_cached_translation(master)

        for form in self.forms:
            set_cached_translation(master, form.instance)
            exclusions = form._get_validation_exclusions()
            # fields from the shared model should not be validated
            exclusions.extend(f.name for f in master._meta.fields)
            try:
                master.clean()
            except ValidationError as e:
                form._update_errors(e)

        set_cached_translation(master, stashed)

        # Validate that at least one translation exists
        forms_to_delete = self.deleted_forms
        provided = [form for form in self.forms
                    if (getattr(form.instance, 'pk', None) is not None or
                        form.has_changed())
                       and not form in forms_to_delete]
        if len(provided) < 1:
            raise ValidationError(_('At least one translation must be provided'),
                                  code='notranslation')

    def _save_translation(self, form, commit=True):
        obj = form.save(commit=False)
        assert isinstance(obj, BaseTranslationModel)

        if commit:
            # We need to trigger custom save actions on the combined model
            stashed = set_cached_translation(self.instance, obj)
            self.instance.save()
            if hasattr(obj, 'save_m2m'): # pragma: no cover
                # cannot happen, but feature could be added, be ready
                obj.save_m2m()
            set_cached_translation(self.instance, stashed)
        return obj

    def save_new(self, form, commit=True):
        return self._save_translation(form, commit)

    def save_existing(self, form, instance, commit=True):
        return self._save_translation(form, commit)

    def add_fields(self, form, index):
        super(BaseTranslationFormSet, self).add_fields(form, index)
        # Add the language code automagically
        if not 'language_code' in form.fields:
            form.fields['language_code'] = CharField(
                required=True, initial=form.instance.language_code,
                widget=Select(choices=(('', '--'),)+settings.LANGUAGES)
            )
            # Add language_code to self._meta.fields so it is included in validation stage
            try:
                form._meta.fields.append('language_code')
            except AttributeError: #pragma: no cover
                form._meta.fields += ('language_code',)

        # Remove the master foreignkey, we have this from self.instance already
        if 'master' in form.fields:
            del form.fields['master']

def translationformset_factory(model, **kwargs):
    """ Works as a regular inlineformset_factory except for:
    - it is set up to work on the given model's translations table
    - it uses a BaseTranslationFormSet to handle combine() and language_code

    Basic use: MyModelTranslationsFormSet = translationformset_factory(MyModel)
    """
    defaults = {
        'formset': BaseTranslationFormSet,
        'fk_name': 'master',
    }
    defaults.update(kwargs)
    return inlineformset_factory(model, model._meta.translations_model, **defaults)




# -*- coding: utf-8 -*-
from rest_framework.serializers import ModelSerializer, CharField
from hvad.test_utils.context_managers import LanguageOverride
from hvad.test_utils.testcase import HvadTestCase
from hvad.test_utils.project.app.models import Normal, Related, TranslatedMany
from hvad.test_utils.data import NORMAL
from hvad.test_utils.fixtures import NormalFixture
from hvad.contrib.restframework import (TranslationsMixin,
                                        TranslatableModelSerializer)
from hvad.contrib.restframework.serializers import TranslationListSerializer

#=============================================================================

class AutoSerializer(TranslatableModelSerializer):
    class Meta:
        model = Normal

class ManualSerializer(TranslatableModelSerializer):
    class Meta:
        model = Normal
        fields = ['shared_field', 'translated_field']

class ExcludeSerializer(TranslatableModelSerializer):
    class Meta:
        model = Normal
        exclude = ['translated_field']

class TranslationsSerializer(TranslationsMixin, ModelSerializer):
    class Meta:
        model = Normal

class CombinedSerializer(TranslationsMixin, TranslatableModelSerializer):
    class Meta:
        model = Normal

class CustomTranslationSerializer(ModelSerializer):
    # 'cheat' tests that shared fields are accessible to the translation serializer
    # It is relevant, it ensures custom serializers see the full object, along with
    # any @property. Default serializer will just get to translated fields through
    # their accessors on the shared object and work transparently.
    cheat = CharField(max_length=250, source='shared_field')
    custom = CharField(max_length=250, source='translated_field')
    class Meta:
        exclude = ('translated_field',)

class CustomSerializer(TranslationsMixin, ModelSerializer):
    class Meta:
        model = Normal
        translations_serializer = CustomTranslationSerializer

class RelatedSerializer(TranslatableModelSerializer):
    class Meta:
        model = Related

class RelatedTranslationsSerializer(TranslationsMixin, ModelSerializer):
    class Meta:
        model = Related

#=============================================================================

class TranslatableModelSerializerTests(HvadTestCase, NormalFixture):
    'Checking the serializer representation of objects'
    normal_count = 2

    #---------------------------------------------------------------------

    def test_modelserializer_fields(self):
        'Check serializers fields are properly set'
        serializer = AutoSerializer()
        self.assertCountEqual(serializer.fields,
                              ['id', 'shared_field', 'translated_field', 'language_code'])

        serializer = ManualSerializer()
        self.assertCountEqual(serializer.fields,
                              ['shared_field', 'translated_field'])

        serializer = ExcludeSerializer()
        self.assertCountEqual(serializer.fields,
                              ['id', 'shared_field', 'language_code'])

    #---------------------------------------------------------------------

    def test_serialize_normal(self):
        'Serialize translated fields using instance language'
        obj = Normal.objects.language('ja').get(pk=self.normal_id[1])

        serializer = AutoSerializer(instance=obj)
        data = serializer.data
        self.assertCountEqual(data, ['id', 'shared_field', 'translated_field', 'language_code'])
        self.assertEqual(data['id'], self.normal_id[1])
        self.assertEqual(data['shared_field'], NORMAL[1].shared_field)
        self.assertEqual(data['translated_field'], NORMAL[1].translated_field['ja'])
        self.assertEqual(data['language_code'], 'ja')

    def test_serialize_enforce_wrong(self):
        'Serialize translated fields while enforcing a language - wrong translation'
        obj = Normal.objects.language('ja').get(pk=self.normal_id[1])

        serializer = AutoSerializer(instance=obj, language='en')
        data = serializer.data
        self.assertCountEqual(data, ['id', 'shared_field', 'translated_field', 'language_code'])
        self.assertEqual(data['id'], self.normal_id[1])
        self.assertEqual(data['shared_field'], NORMAL[1].shared_field)
        self.assertEqual(data['translated_field'], NORMAL[1].translated_field['en'])
        self.assertEqual(data['language_code'], 'en')

    def test_serialize_enforce_nonexistent(self):
        'Serialize translated fields while enforcing a language - nonexistent translation'
        obj = Normal.objects.language('ja').get(pk=self.normal_id[1])

        serializer = AutoSerializer(instance=obj, language='xx')
        data = serializer.data
        self.assertCountEqual(data, ['id', 'shared_field', 'translated_field', 'language_code'])
        self.assertEqual(data['id'], self.normal_id[1])
        self.assertEqual(data['shared_field'], NORMAL[1].shared_field)
        self.assertEqual(data['translated_field'], '')
        self.assertEqual(data['language_code'], 'xx')

    def test_serialize_related(self):
        'Serialize relation fields'
        obj = Related(pk=42, language_code='ja',
                      normal_id=self.normal_id[1], translated_id=self.normal_id[2])

        serializer = RelatedSerializer(instance=obj)
        data = serializer.data

        self.assertCountEqual(data, ['id', 'normal', 'language_code',
                                     'translated', 'translated_to_translated'])
        self.assertEqual(data['id'], 42)
        self.assertEqual(data['normal'], self.normal_id[1])
        self.assertEqual(data['language_code'], 'ja')
        self.assertEqual(data['translated'], self.normal_id[2])
        self.assertEqual(data['translated_to_translated'], None)

    #---------------------------------------------------------------------

    def test_create_normal(self):
        'Deserialize a new instance'
        data = {
            'shared_field': 'shared',
            'translated_field': 'translated',
            'language_code': 'en'
        }
        serializer = AutoSerializer(data=data)
        self.assertTrue(serializer.is_valid())

        obj = serializer.save()
        self.assertIsNotNone(obj.pk)
        self.assertSavedObject(obj, 'en', **data)

    def test_create_enforce(self):
        'Deserialize a new instance, with language-enforcing mode'
        data = {
            'shared_field': 'shared',
            'translated_field': 'translated',
        }
        serializer = AutoSerializer(data=data, language='sr')
        self.assertTrue(serializer.is_valid())

        obj = serializer.save()
        self.assertIsNotNone(obj.pk)
        self.assertSavedObject(obj, 'sr', **data)

    def test_create_enforce_violation(self):
        'Deserialize a new instance, with language-enforcing mode and language_code'
        data = {
            'shared_field': 'shared',
            'translated_field': 'translated',
            'language_code': 'en',
        }
        serializer = AutoSerializer(data=data, language='en')
        self.assertFalse(serializer.is_valid())
        serializer = AutoSerializer(data=data, language='xx')
        self.assertFalse(serializer.is_valid())

    def test_update_normal_default(self):
        'Deserialize an existing instance using instance-loaded language'
        obj = Normal.objects.language('ja').get(pk=self.normal_id[1])
        data = {
            'shared_field': 'shared',
            'translated_field': 'translated',
        }
        serializer = AutoSerializer(instance=obj, data=data)
        self.assertTrue(serializer.is_valid())

        obj = serializer.save()
        self.assertEqual(obj.pk, self.normal_id[1])
        self.assertSavedObject(obj, 'ja', **data)

        obj = Normal.objects.untranslated().get(pk=self.normal_id[1])
        serializer = AutoSerializer(instance=obj, data=data)
        self.assertTrue(serializer.is_valid())
        with LanguageOverride('en'):
            obj = serializer.save()
        self.assertEqual(obj.pk, self.normal_id[1])
        self.assertSavedObject(obj, 'en', **data)

    def test_update_normal_language_code(self):
        'Deserialize an existing instance using submitted language'
        obj = Normal.objects.language('ja').get(pk=self.normal_id[1])
        data = {
            'shared_field': 'shared',
            'translated_field': 'translated',
            'language_code': 'sr'
        }
        serializer = AutoSerializer(instance=obj, data=data)
        self.assertTrue(serializer.is_valid())

        obj = serializer.save()
        self.assertEqual(obj.pk, self.normal_id[1])
        self.assertSavedObject(obj, 'sr', **data)

        obj = Normal.objects.untranslated().get(pk=self.normal_id[1])
        data['translated_field'] = 'translated_bis'
        serializer = AutoSerializer(instance=obj, data=data)
        self.assertTrue(serializer.is_valid())
        with LanguageOverride('en'):
            obj = serializer.save()
        self.assertEqual(obj.pk, self.normal_id[1])
        self.assertSavedObject(obj, 'sr', **data)

    def test_update_enforce(self):
        'Deserialize an existing intance in language-enforcing mode'
        data = {
            'shared_field': 'shared',
            'translated_field': 'translated',
        }
        # Correct translation
        obj = Normal.objects.language('ja').get(pk=self.normal_id[1])
        serializer = AutoSerializer(instance=obj, data=data, language='ja')
        self.assertTrue(serializer.is_valid())

        obj = serializer.save()
        self.assertEqual(obj.pk, self.normal_id[1])
        self.assertSavedObject(obj, 'ja', **data)

        # Wrong translation
        obj = Normal.objects.language('en').get(pk=self.normal_id[1])
        serializer = AutoSerializer(instance=obj, data=data, language='ja')
        self.assertTrue(serializer.is_valid())

        obj = serializer.save()
        self.assertEqual(obj.pk, self.normal_id[1])
        self.assertSavedObject(obj, 'ja', **data)

        # Nonexistent translation
        obj = Normal.objects.language('en').get(pk=self.normal_id[1])
        serializer = AutoSerializer(instance=obj, data=data, language='sr')
        self.assertTrue(serializer.is_valid())

        obj = serializer.save()
        self.assertEqual(obj.pk, self.normal_id[1])
        self.assertSavedObject(obj, 'sr', **data)

#=============================================================================

class TranslationsMixinTests(HvadTestCase, NormalFixture):
    normal_count = 2

    def test_translations_mixin_fields(self):
        'Check serializers fields are properly set'
        serializer = TranslationsSerializer()
        self.assertCountEqual(serializer.fields,
                              ['id', 'shared_field', 'translations'])
        self.assertIsInstance(serializer.fields['translations'], TranslationListSerializer)
        self.assertCountEqual(serializer.fields['translations'].child.fields,
                              ['translated_field'])

        serializer = CustomSerializer()
        self.assertCountEqual(serializer.fields,
                              ['id', 'shared_field', 'translations'])
        self.assertIsInstance(serializer.fields['translations'], TranslationListSerializer)
        self.assertIsInstance(serializer.fields['translations'].child, CustomTranslationSerializer)
        self.assertCountEqual(serializer.fields['translations'].child.fields, ['cheat', 'custom'])

    #---------------------------------------------------------------------

    def test_serialize(self):
        'Serialize nested translations as a language => fields dict'
        obj = Normal.objects.prefetch_related('translations').get(pk=self.normal_id[1])

        serializer = TranslationsSerializer(instance=obj)
        data = serializer.data
        self.assertCountEqual(data, ['id', 'shared_field', 'translations'])
        self.assertEqual(data['id'], self.normal_id[1])
        self.assertEqual(data['shared_field'], NORMAL[1].shared_field)
        self.assertIsInstance(data['translations'], dict)
        self.assertCountEqual(data['translations'], self.translations)
        for language in self.translations:
            translation = data['translations'][language]
            self.assertCountEqual(translation, ['translated_field'])
            self.assertEqual(translation['translated_field'], NORMAL[1].translated_field[language])

    def test_serialize_custom(self):
        'Serialize nested translations as a language => fields dict'
        obj = Normal.objects.prefetch_related('translations').get(pk=self.normal_id[1])

        serializer = CustomSerializer(instance=obj)
        data = serializer.data
        self.assertCountEqual(data, ['id', 'shared_field', 'translations'])
        self.assertEqual(data['id'], self.normal_id[1])
        self.assertEqual(data['shared_field'], NORMAL[1].shared_field)
        self.assertIsInstance(data['translations'], dict)
        self.assertCountEqual(data['translations'], self.translations)
        for language in self.translations:
            translation = data['translations'][language]
            self.assertCountEqual(translation, ['cheat', 'custom'])
            self.assertEqual(translation['cheat'], NORMAL[1].shared_field)
            self.assertEqual(translation['custom'], NORMAL[1].translated_field[language])

    def test_serialize_related(self):
        'Serialize relation fields'
        obj = Related.objects.language('ja').create(
            normal_id=self.normal_id[1], translated_id=self.normal_id[2]
        )

        serializer = RelatedTranslationsSerializer(instance=obj)
        data = serializer.data

        self.assertCountEqual(data, ['id', 'normal', 'translated_fields'])
        self.assertEqual(data['id'], obj.pk)
        self.assertEqual(data['normal'], self.normal_id[1])
        self.assertIsInstance(data['translated_fields'], dict)
        for language in ['ja']:
            translation = data['translated_fields'][language]
            self.assertCountEqual(translation, ['translated', 'translated_to_translated'])
            self.assertEqual(translation['translated'], self.normal_id[2])
            self.assertEqual(translation['translated_to_translated'], None)

    #---------------------------------------------------------------------

    def test_invalid(self):
        'Submit invalid data'
        # No translations
        data = {
            'shared_field': 'shared',
            'translations': {},
        }
        serializer = TranslationsSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertTrue(serializer.errors['translations'])

        # Invalid translations type
        data = {
            'shared_field': 'shared',
            'translations': [
                { 'translated_field': 'English', },
            ],
        }
        serializer = TranslationsSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertTrue(serializer.errors['translations'])

        # Cascade invalid child
        data = {
            'shared_field': 'shared',
            'translations': {
                'en': { 'translated_field': 'x'*999 },
            },
        }
        serializer = TranslationsSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertTrue(serializer.errors['translations'])
        self.assertTrue(serializer.errors['translations']['en'])
        self.assertTrue(serializer.errors['translations']['en']['translated_field'])

    #---------------------------------------------------------------------

    def test_create(self):
        'Create a new Normal instance, with two translations'
        data = {
            'shared_field': 'shared',
            'translations': {
                'en': { 'translated_field': 'English', },
                'sr': { 'translated_field': u'српски', },
            },
        }
        serializer = TranslationsSerializer(data=data)
        self.assertTrue(serializer.is_valid())

        with self.assertNumQueries(3):  # insert shared, insert "en", insert "sr"
            obj = serializer.save()
        self.assertIsNot(obj.pk, None)
        qs = Normal.objects.language('all').filter(pk=obj.pk)
        self.assertCountEqual([(obj.language_code, obj.translated_field) for obj in qs],
                              [('en', 'English'), ('sr', u'српски')])

    def test_update(self):
        'Update an existing normal instance: 1 new, 1 updated, 1 deleted translations'
        obj = Normal.objects.untranslated().prefetch_related('translations').get(pk=self.normal_id[1])
        data = {
            'shared_field': 'shared',
            'translations': {
                'en': { 'translated_field': 'English', }, # should updated
                'sr': { 'translated_field': u'српски', }, # should create
            },                                            # Japanese should be deleted
        }
        serializer = TranslationsSerializer(instance=obj, data=data)
        self.assertTrue(serializer.is_valid())

        with self.assertNumQueries(4):  # update shared, update "en", insert "sr", delete others
            obj = serializer.save()
        self.assertEqual(obj.pk, self.normal_id[1])
        qs = Normal.objects.language('all').filter(pk=self.normal_id[1])
        self.assertCountEqual([(obj.language_code, obj.translated_field) for obj in qs],
                              [('en', 'English'), ('sr', u'српски')])

    def test_update_partial(self):
        'Update an existing instance, but just some fields'
        obj = Normal.objects.untranslated().get(pk=self.normal_id[1])
        data = {
            'shared_field': 'shared'
        }
        serializer = TranslationsSerializer(instance=obj, data=data, partial=True)
        self.assertTrue(serializer.is_valid())

        with self.assertNumQueries(1):  # update shared
            obj = serializer.save()
        self.assertEqual(obj.pk, self.normal_id[1])
        qs = Normal.objects.language('all').filter(pk=self.normal_id[1], shared_field='shared')
        self.assertCountEqual([obj.language_code for obj in qs], self.translations)

    def test_update_translated_many(self):
        'Update an existing instance, with an excluded translated M2M field'
        """ WARNING - this is unsupported, this test case is here so we know
            what we break rather than to enforce that behavior
        """
        obj = TranslatedMany.objects.language('en').create(name='shared', translated_field='English')
        # reload with normal caching conditions so query counting works
        obj = TranslatedMany.objects.untranslated().prefetch_related('translations').get(pk=obj.pk)
        data = {
            'name': 'shared-updated',
            'translations': {
                'en': {'translated_field': u'English-updated'},
                'fr': {'translated_field': u'French-added'},
            },
        }
        class TranslationSerializerClass(ModelSerializer):
            class Meta:
                exclude = ['many']
        class SerializerClass(TranslationsMixin, ModelSerializer):
            class Meta:
                model = TranslatedMany
                translations_serializer = TranslationSerializerClass

        serializer = SerializerClass(instance=obj, data=data)
        self.assertTrue(serializer.is_valid())
        with self.assertNumQueries(4):  # update shared, update en, insert fr
                                        # select to delete (none so no actual delete after)
            serializer.save()

#=============================================================================

class CombinedTests(HvadTestCase, NormalFixture):
    normal_count = 1

    def test_combined_fields(self):
        'Check serializers fields are properly set'
        serializer = CombinedSerializer()
        self.assertCountEqual(serializer.fields,
                              ['id', 'shared_field', 'translated_field', 'language_code', 'translations'])
        self.assertIsInstance(serializer.fields['translations'], TranslationListSerializer)
        self.assertCountEqual(serializer.fields['translations'].child.fields,
                              ['translated_field'])

    #---------------------------------------------------------------------

    def test_serialize(self):
        'Serialize translations as a language => fields dict + naive fields'
        obj = Normal.objects.language('ja').prefetch_related('translations').get(pk=self.normal_id[1])

        serializer = CombinedSerializer(instance=obj)
        data = serializer.data
        self.assertCountEqual(data, ['id', 'shared_field', 'translated_field', 'language_code', 'translations'])
        self.assertEqual(data['id'], self.normal_id[1])
        self.assertEqual(data['shared_field'], NORMAL[1].shared_field)
        self.assertEqual(data['translated_field'], NORMAL[1].translated_field['ja'])
        self.assertEqual(data['language_code'], 'ja')
        self.assertIsInstance(data['translations'], dict)
        self.assertCountEqual(data['translations'], self.translations)
        for language in self.translations:
            translation = data['translations'][language]
            self.assertCountEqual(translation, ['translated_field'])
            self.assertEqual(translation['translated_field'], NORMAL[1].translated_field[language])

    #---------------------------------------------------------------------

    def test_create_translations(self):
        'Create a new Normal instance, with two translations'
        data = {
            'shared_field': 'shared',
            'translated_field': 'should be ignored',
            'language_code': 'sr',
            'translations': {
                'en': { 'translated_field': 'English', },
                'sr': { 'translated_field': u'српски', },
            },
        }
        serializer = CombinedSerializer(data=data)
        self.assertTrue(serializer.is_valid())

        obj = serializer.save()
        self.assertIsNot(obj.pk, None)
        qs = Normal.objects.language('all').filter(pk=obj.pk)
        self.assertCountEqual([(obj.language_code, obj.translated_field) for obj in qs],
                              [('en', 'English'), ('sr', u'српски')])

    def test_create_translatable(self):
        'Create a new Normal instance, in translatablemodelserializer style'
        data = {
            'shared_field': 'shared',
            'translated_field': u'српски',
            'language_code': 'sr'
        }
        serializer = CombinedSerializer(data=data)
        self.assertTrue(serializer.is_valid())

        obj = serializer.save()
        self.assertIsNot(obj.pk, None)
        qs = Normal.objects.language('all').filter(pk=obj.pk)
        self.assertCountEqual([(obj.language_code, obj.translated_field) for obj in qs],
                              [('sr', u'српски')])

    def test_update_translations(self):
        'Update an existing normal instance: 1 new, 1 updated, 1 deleted translations'
        obj = Normal.objects.untranslated().get(pk=self.normal_id[1])
        data = {
            'shared_field': 'shared',
            'language_code': 'ignored',
            'translations': {
                'en': { 'translated_field': 'English', }, # should updated
                'sr': { 'translated_field': u'српски', }, # should create
            },                                            # Japanese should be deleted
        }
        serializer = CombinedSerializer(instance=obj, data=data)
        self.assertTrue(serializer.is_valid())

        obj = serializer.save()
        self.assertEqual(obj.pk, self.normal_id[1])
        qs = Normal.objects.language('all').filter(pk=self.normal_id[1])
        self.assertCountEqual([(obj.language_code, obj.translated_field) for obj in qs],
                              [('en', 'English'), ('sr', u'српски')])

    def test_update_translatable(self):
        'Update an existing normal instance translation in translatablemodel mode'
        obj = Normal.objects.untranslated().get(pk=self.normal_id[1])
        data = {
            'shared_field': 'shared',
            'translated_field': u'српски',
            'language_code': 'sr'
        }
        serializer = CombinedSerializer(instance=obj, data=data)
        self.assertTrue(serializer.is_valid())

        obj = serializer.save()
        self.assertEqual(obj.pk, self.normal_id[1])
        qs = Normal.objects.language('all').filter(pk=self.normal_id[1])
        self.assertCountEqual([(obj.language_code, obj.translated_field) for obj in qs],
                              [('en', NORMAL[1].translated_field['en']),
                               ('ja', NORMAL[1].translated_field['ja']),
                               ('sr', u'српски')])



from setuptools import setup, find_packages
from hvad import __version__ as version

with open('README.rst', 'rb') as f:
    long_description = f.read().decode('utf-8')

setup(
    name = 'django-hvad',
    version = version,
    description = 'A content translation framework for django integrated automatically in the normal ORM. Removes the pain of having to think about translations in a django project.',
    long_description = long_description,
    author = 'Kristian Ollegaard',
    author_email = 'kristian.ollegaard@divio.ch',
    url = 'https://github.com/KristianOellegaard/django-hvad',
    packages = find_packages(
        exclude = [
            'testproject',
            'testproject.app',
            'hvad.tests',
        ],
    ),
    zip_safe=False,
    include_package_data = True,
    install_requires=[
        'Django>=1.7',
    ],
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Database",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Text Processing :: Linguistic",
    ]
)



import django
from django.db.models.fields import FieldDoesNotExist
from django.utils.translation import get_language
from hvad.utils import get_translation, set_cached_translation, get_cached_translation
if django.VERSION >= (1, 7):
    from django.apps import registry

class BaseDescriptor(object):
    """
    Base descriptor class with a helper to get the translations instance.
    """
    def __init__(self, opts):
        self.opts = opts
        self._NoTranslationError = type('NoTranslationError',
                                        (AttributeError, opts.translations_model.DoesNotExist),
                                        {})
    
    def translation(self, instance):
        translation = get_cached_translation(instance)
        if translation is None:
            try:
                translation = get_translation(instance)
            except self.opts.translations_model.DoesNotExist:
                raise self._NoTranslationError('Accessing a translated field requires that '
                                               'the instance has a translation loaded, or a '
                                               'valid translation in current language (%s) '
                                               'loadable from the database' % get_language())
            set_cached_translation(instance, translation)
        return translation


class TranslatedAttribute(BaseDescriptor):
    """ Proxies attributes from the shared instance to the translated instance. """

    def __init__(self, opts, name):
        self.name = name
        super(TranslatedAttribute, self).__init__(opts)

    def __get__(self, instance, instance_type=None):
        if not instance:
            if django.VERSION >= (1, 7) and not registry.apps.ready: #pragma: no cover
                raise AttributeError('Attribute not available until registry is ready.')
            # Don't raise an attribute error so we can use it in admin.
            try:
                if django.VERSION >= (1, 8):
                    return self.opts.translations_model._meta.get_field(self.name).default
                else:
                    return self.opts.translations_model._meta.get_field_by_name(self.name)[0].default
            except FieldDoesNotExist as e: #pragma: no cover (django 1.6 before models loaded)
                raise AttributeError(*e.args)
        return getattr(self.translation(instance), self.name)
    
    def __set__(self, instance, value):
        setattr(self.translation(instance), self.name, value)
    
    def __delete__(self, instance):
        delattr(self.translation(instance), self.name)


class LanguageCodeAttribute(TranslatedAttribute):
    """
    The language_code attribute is different from other attribtues as it cannot
    be deleted. Trying to do so will always cause an attribute error.
    
    """
    def __init__(self, opts):
        super(LanguageCodeAttribute, self).__init__(opts, 'language_code')
    
    def __set__(self, instance, value):
        raise AttributeError("The 'language_code' attribute cannot be "
                             "changed directly! Use the translate() method instead.")
    
    def __delete__(self, instance):
        raise AttributeError("The 'language_code' attribute cannot be deleted!")
